@@ -1,4 +1,4 @@
-/* $Id: ai_depotlist.hpp 17248 2009-08-21 20:21:05Z rubidium $ */
+/* $Id: ai_depotlist.hpp 20563 2010-08-19 15:37:28Z yexo $ */
 
 /*
  * This file is part of OpenTTD.
@@ -12,15 +12,16 @@
 #ifndef AI_DEPOTLIST_HPP
 #define AI_DEPOTLIST_HPP
 
-#include "ai_abstractlist.hpp"
+#include "ai_list.hpp"
 #include "ai_tile.hpp"
 
 /**
  * Creates a list of the locations of the depots (and hangars) of which you are the owner.
  * @ingroup AIList
  */
-class AIDepotList : public AIAbstractList {
+class AIDepotList : public AIList {
 public:
+	/** Get the name of this class to identify it towards squirrel. */
 	static const char *GetClassName() { return "AIDepotList"; }
 
 	/**
