@@ -1,4 +1,4 @@
-/* $Id: console_cmds.cpp 20520 2010-08-16 23:06:22Z rubidium $ */
+/* $Id: console_cmds.cpp 22838 2011-08-25 13:24:32Z rubidium $ */
 
 /*
  * This file is part of OpenTTD.
@@ -18,6 +18,8 @@
 #include "network/network.h"
 #include "network/network_func.h"
 #include "network/network_base.h"
+#include "network/network_admin.h"
+#include "network/network_client.h"
 #include "command_func.h"
 #include "settings_func.h"
 #include "fios.h"
@@ -28,25 +30,24 @@
 #include "viewport_func.h"
 #include "window_func.h"
 #include "date_func.h"
-#include "vehicle_func.h"
 #include "company_func.h"
-#include "company_base.h"
 #include "gamelog.h"
 #include "ai/ai.hpp"
 #include "ai/ai_config.hpp"
+#include "newgrf.h"
 #include "console_func.h"
+#include "engine_base.h"
 
 #ifdef ENABLE_NETWORK
 	#include "table/strings.h"
 #endif /* ENABLE_NETWORK */
 
 /* scriptfile handling */
-static FILE *_script_file;
-static bool _script_running;
+static bool _script_running; ///< Script is running (used to abort execution when #ConReturn is encountered).
 
 /* console command defines */
 #define DEF_CONSOLE_CMD(function) static bool function(byte argc, char *argv[])
-#define DEF_CONSOLE_HOOK(function) static bool function()
+#define DEF_CONSOLE_HOOK(function) static ConsoleHookResult function(bool echo)
 
 
 /****************
@@ -55,66 +56,110 @@ static bool _script_running;
 
 #ifdef ENABLE_NETWORK
 
-static inline bool NetworkAvailable()
+/**
+ * Check network availability and inform in console about failure of detection.
+ * @return Network availability.
+ */
+static inline bool NetworkAvailable(bool echo)
 {
 	if (!_network_available) {
-		IConsoleError("You cannot use this command because there is no network available.");
+		if (echo) IConsoleError("You cannot use this command because there is no network available.");
 		return false;
 	}
 	return true;
 }
 
+/**
+ * Check whether we are a server.
+ * @return Are we a server? True when yes, false otherwise.
+ */
 DEF_CONSOLE_HOOK(ConHookServerOnly)
 {
-	if (!NetworkAvailable()) return false;
+	if (!NetworkAvailable(echo)) return CHR_DISALLOW;
 
 	if (!_network_server) {
-		IConsoleError("This command is only available to a network server.");
-		return false;
+		if (echo) IConsoleError("This command is only available to a network server.");
+		return CHR_DISALLOW;
 	}
-	return true;
+	return CHR_ALLOW;
 }
 
+/**
+ * Check whether we are a client in a network game.
+ * @return Are we a client in a network game? True when yes, false otherwise.
+ */
 DEF_CONSOLE_HOOK(ConHookClientOnly)
 {
-	if (!NetworkAvailable()) return false;
+	if (!NetworkAvailable(echo)) return CHR_DISALLOW;
 
 	if (_network_server) {
-		IConsoleError("This command is not available to a network server.");
-		return false;
+		if (echo) IConsoleError("This command is not available to a network server.");
+		return CHR_DISALLOW;
 	}
-	return true;
+	return CHR_ALLOW;
 }
 
+/**
+ * Check whether we are in a multiplayer game.
+ * @return True when we are client or server in a network game.
+ */
 DEF_CONSOLE_HOOK(ConHookNeedNetwork)
 {
-	if (!NetworkAvailable()) return false;
+	if (!NetworkAvailable(echo)) return CHR_DISALLOW;
 
-	if (!_networking) {
-		IConsoleError("Not connected. This command is only available in multiplayer.");
-		return false;
+	if (!_networking || (!_network_server && !MyClient::IsConnected())) {
+		if (echo) IConsoleError("Not connected. This command is only available in multiplayer.");
+		return CHR_DISALLOW;
 	}
-	return true;
+	return CHR_ALLOW;
 }
 
+/**
+ * Check whether we are in single player mode.
+ * @return True when no network is active.
+ */
 DEF_CONSOLE_HOOK(ConHookNoNetwork)
 {
 	if (_networking) {
-		IConsoleError("This command is forbidden in multiplayer.");
-		return false;
+		if (echo) IConsoleError("This command is forbidden in multiplayer.");
+		return CHR_DISALLOW;
 	}
-	return true;
+	return CHR_ALLOW;
 }
 
 #else
 #	define ConHookNoNetwork NULL
 #endif /* ENABLE_NETWORK */
 
+DEF_CONSOLE_HOOK(ConHookNewGRFDeveloperTool)
+{
+	if (_settings_client.gui.newgrf_developer_tools) {
+		if (_game_mode == GM_MENU) {
+			if (echo) IConsoleError("This command is only available in game and editor.");
+			return CHR_DISALLOW;
+		}
+#ifdef ENABLE_NETWORK
+		return ConHookNoNetwork(echo);
+#else
+		return CHR_ALLOW;
+#endif
+	}
+	return CHR_HIDE;
+}
+
+/**
+ * Show help for the console.
+ * @param str String to print in the console.
+ */
 static void IConsoleHelp(const char *str)
 {
 	IConsolePrintF(CC_WARNING, "- %s", str);
 }
 
+/**
+ * Reset status of all engines.
+ * @return Will always succeed.
+ */
 DEF_CONSOLE_CMD(ConResetEngines)
 {
 	if (argc == 0) {
@@ -126,7 +171,37 @@ DEF_CONSOLE_CMD(ConResetEngines)
 	return true;
 }
 
+/**
+ * Reset status of the engine pool.
+ * @return Will always return true.
+ * @note Resetting the pool only succeeds when there are no vehicles ingame.
+ */
+DEF_CONSOLE_CMD(ConResetEnginePool)
+{
+	if (argc == 0) {
+		IConsoleHelp("Reset NewGRF allocations of engine slots. This will remove invalid engine definitions, and might make default engines available again.");
+		return true;
+	}
+
+	if (_game_mode == GM_MENU) {
+		IConsoleError("This command is only available in game and editor.");
+		return true;
+	}
+
+	if (!EngineOverrideManager::ResetToCurrentNewGRFConfig()) {
+		IConsoleError("This can only be done when there are no vehicles in the game.");
+		return true;
+	}
+
+	return true;
+}
+
 #ifdef _DEBUG
+/**
+ * Reset a tile to bare land in debug mode.
+ * param tile number.
+ * @return True when the tile is reset or the help on usage was printed (0 or two parameters).
+ */
 DEF_CONSOLE_CMD(ConResetTile)
 {
 	if (argc == 0) {
@@ -145,19 +220,17 @@ DEF_CONSOLE_CMD(ConResetTile)
 
 	return false;
 }
-
-DEF_CONSOLE_CMD(ConStopAllVehicles)
-{
-	if (argc == 0) {
-		IConsoleHelp("Stops all vehicles in the game. For debugging only! Use at your own risk... Usage: 'stopall'");
-		return true;
-	}
-
-	StopAllVehicles();
-	return true;
-}
 #endif /* _DEBUG */
 
+/**
+ * Scroll to a tile on the map.
+ * @param arg1 tile tile number or tile x coordinate.
+ * @param arg2 optionally tile y coordinate.
+ * @note When only one argument is given it is intepreted as the tile number.
+ *       When two arguments are given, they are interpreted as the tile's x
+ *       and y coordinates.
+ * @return True when either console help was shown or a proper amount of parameters given.
+ */
 DEF_CONSOLE_CMD(ConScrollToTile)
 {
 	switch (argc) {
@@ -197,10 +270,11 @@ DEF_CONSOLE_CMD(ConScrollToTile)
 	return false;
 }
 
-extern void BuildFileList();
-extern void SetFiosType(const byte fiostype);
-
-/* Save the map to a file */
+/**
+ * Save the map to a file.
+ * @param filename the filename to save the map to.
+ * @return True when help was displayed or the file attempted to be saved.
+ */
 DEF_CONSOLE_CMD(ConSave)
 {
 	if (argc == 0) {
@@ -224,11 +298,15 @@ DEF_CONSOLE_CMD(ConSave)
 	return false;
 }
 
-/* Explicitly save the configuration */
+/**
+ * Explicitly save the configuration.
+ * @return True.
+ */
 DEF_CONSOLE_CMD(ConSaveConfig)
 {
 	if (argc == 0) {
-		IConsoleHelp("Saves the current config, typically to 'openttd.cfg'.");
+		IConsoleHelp("Saves the configuration for new games to the configuration file, typically 'openttd.cfg'.");
+		IConsoleHelp("It does not save the configuration of the current game to the configuration file.");
 		return true;
 	}
 
@@ -237,6 +315,12 @@ DEF_CONSOLE_CMD(ConSaveConfig)
 	return true;
 }
 
+/**
+ * Get savegame file informations.
+ * @param file The savegame filename to return information about. Can be the actual name
+ *             or a numbered entry into the filename list.
+ * @return FiosItem The information on the file.
+ */
 static const FiosItem *GetFiosItem(const char *file)
 {
 	_saveload_mode = SLD_LOAD_GAME;
@@ -286,7 +370,8 @@ DEF_CONSOLE_CMD(ConLoad)
 
 				strecpy(_file_to_saveload.name, FiosBrowseTo(item), lastof(_file_to_saveload.name));
 				strecpy(_file_to_saveload.title, item->title, lastof(_file_to_saveload.title));
-			} break;
+				break;
+			}
 			default: IConsolePrintF(CC_ERROR, "%s: Not a savegame.", file);
 		}
 	} else {
@@ -310,8 +395,9 @@ DEF_CONSOLE_CMD(ConRemove)
 	const char *file = argv[1];
 	const FiosItem *item = GetFiosItem(file);
 	if (item != NULL) {
-		if (!FiosDelete(item->name))
+		if (!FiosDelete(item->name)) {
 			IConsolePrintF(CC_ERROR, "%s: Failed to delete file", file);
+		}
 	} else {
 		IConsolePrintF(CC_ERROR, "%s: No such file or directory.", file);
 	}
@@ -404,17 +490,21 @@ DEF_CONSOLE_CMD(ConClearBuffer)
 
 static bool ConKickOrBan(const char *argv, bool ban)
 {
-	const char *ip = argv;
+	uint n;
 
 	if (strchr(argv, '.') == NULL && strchr(argv, ':') == NULL) { // banning with ID
 		ClientID client_id = (ClientID)atoi(argv);
 
-		if (client_id == CLIENT_ID_SERVER) {
+		/* Don't kill the server, or the client doing the rcon. The latter can't be kicked because
+		 * kicking frees closes and subsequently free the connection related instances, which we
+		 * would be reading from and writing to after returning. So we would read or write data
+		 * from freed memory up till the segfault triggers. */
+		if (client_id == CLIENT_ID_SERVER || client_id == _redirect_console_to_client) {
 			IConsolePrintF(CC_ERROR, "ERROR: Silly boy, you can not %s yourself!", ban ? "ban" : "kick");
 			return true;
 		}
 
-		NetworkClientInfo *ci = NetworkFindClientInfoFromClientID(client_id);
+		NetworkClientInfo *ci = NetworkClientInfo::GetByClientID(client_id);
 		if (ci == NULL) {
 			IConsoleError("Invalid client");
 			return true;
@@ -427,10 +517,11 @@ static bool ConKickOrBan(const char *argv, bool ban)
 		}
 
 		/* When banning, kick+ban all clients with that IP */
-		ip = GetClientIP(ci);
+		n = NetworkServerKickOrBanIP(client_id, ban);
+	} else {
+		n = NetworkServerKickOrBanIP(argv, ban);
 	}
 
-	uint n = NetworkServerKickOrBanIP(ip, ban);
 	if (n == 0) {
 		IConsolePrint(CC_DEFAULT, ban ? "Client not online, address added to banlist" : "Client not found");
 	} else {
@@ -609,7 +700,7 @@ DEF_CONSOLE_CMD(ConClientNickChange)
 		return true;
 	}
 
-	if (NetworkFindClientInfoFromClientID(client_id) == NULL) {
+	if (NetworkClientInfo::GetByClientID(client_id) == NULL) {
 		IConsoleError("Invalid client");
 		return true;
 	}
@@ -637,7 +728,7 @@ DEF_CONSOLE_CMD(ConJoinCompany)
 		return true;
 	}
 
-	if (NetworkFindClientInfoFromClientID(_network_own_client_id)->client_playas == company_id) {
+	if (NetworkClientInfo::GetByClientID(_network_own_client_id)->client_playas == company_id) {
 		IConsoleError("You are already there!");
 		return true;
 	}
@@ -676,7 +767,7 @@ DEF_CONSOLE_CMD(ConMoveClient)
 		return true;
 	}
 
-	const NetworkClientInfo *ci = NetworkFindClientInfoFromClientID((ClientID)atoi(argv[1]));
+	const NetworkClientInfo *ci = NetworkClientInfo::GetByClientID((ClientID)atoi(argv[1]));
 	CompanyID company_id = (CompanyID)(atoi(argv[2]) <= MAX_COMPANIES ? atoi(argv[2]) - 1 : atoi(argv[2]));
 
 	/* check the client exists */
@@ -738,14 +829,14 @@ DEF_CONSOLE_CMD(ConResetCompany)
 		IConsoleError("Cannot remove company: a client is connected to that company.");
 		return false;
 	}
-	const NetworkClientInfo *ci = NetworkFindClientInfoFromClientID(CLIENT_ID_SERVER);
+	const NetworkClientInfo *ci = NetworkClientInfo::GetByClientID(CLIENT_ID_SERVER);
 	if (ci->client_playas == index) {
 		IConsoleError("Cannot remove company: the server is connected to that company.");
 		return true;
 	}
 
 	/* It is safe to remove this company */
-	DoCommandP(0, 2, index, CMD_COMPANY_CTRL);
+	DoCommandP(0, 2 | index << 16, 0, CMD_COMPANY_CTRL);
 	IConsolePrint(CC_DEFAULT, "Company deleted.");
 
 	return true;
@@ -794,7 +885,7 @@ DEF_CONSOLE_CMD(ConNetworkReconnect)
 
 	NetworkClientConnectGame(NetworkAddress(_settings_client.network.last_host, _settings_client.network.last_port), playas);
 	return true;
-};
+}
 
 DEF_CONSOLE_CMD(ConNetworkConnect)
 {
@@ -847,9 +938,6 @@ DEF_CONSOLE_CMD(ConNetworkConnect)
 
 DEF_CONSOLE_CMD(ConExec)
 {
-	char cmdline[ICON_CMDLN_SIZE];
-	char *cmdptr;
-
 	if (argc == 0) {
 		IConsoleHelp("Execute a local script file. Usage: 'exec <script> <?>'");
 		return true;
@@ -857,18 +945,19 @@ DEF_CONSOLE_CMD(ConExec)
 
 	if (argc < 2) return false;
 
-	_script_file = FioFOpenFile(argv[1], "r", BASE_DIR);
+	FILE *script_file = FioFOpenFile(argv[1], "r", BASE_DIR);
 
-	if (_script_file == NULL) {
+	if (script_file == NULL) {
 		if (argc == 2 || atoi(argv[2]) != 0) IConsoleError("script file not found");
 		return true;
 	}
 
 	_script_running = true;
 
-	while (_script_running && fgets(cmdline, sizeof(cmdline), _script_file) != NULL) {
+	char cmdline[ICON_CMDLN_SIZE];
+	while (_script_running && fgets(cmdline, sizeof(cmdline), script_file) != NULL) {
 		/* Remove newline characters from the executing script */
-		for (cmdptr = cmdline; *cmdptr != '\0'; cmdptr++) {
+		for (char *cmdptr = cmdline; *cmdptr != '\0'; cmdptr++) {
 			if (*cmdptr == '\n' || *cmdptr == '\r') {
 				*cmdptr = '\0';
 				break;
@@ -877,11 +966,12 @@ DEF_CONSOLE_CMD(ConExec)
 		IConsoleCmdExec(cmdline);
 	}
 
-	if (ferror(_script_file))
+	if (ferror(script_file)) {
 		IConsoleError("Encountered errror while trying to read from script file");
+	}
 
 	_script_running = false;
-	FioFCloseFile(_script_file);
+	FioFCloseFile(script_file);
 	return true;
 }
 
@@ -943,7 +1033,7 @@ DEF_CONSOLE_CMD(ConEchoC)
 	}
 
 	if (argc < 3) return false;
-	IConsolePrint((ConsoleColour)atoi(argv[1]), argv[2]);
+	IConsolePrint((TextColour)Clamp(atoi(argv[1]), TC_BEGIN, TC_END - 1), argv[2]);
 	return true;
 }
 
@@ -955,7 +1045,7 @@ DEF_CONSOLE_CMD(ConNewGame)
 		return true;
 	}
 
-	StartNewGameWithoutGUI((argc == 2) ? strtoul(argv[1], NULL, 10) : (uint)GENERATE_NEW_SEED);
+	StartNewGameWithoutGUI((argc == 2) ? strtoul(argv[1], NULL, 10) : GENERATE_NEW_SEED);
 	return true;
 }
 
@@ -980,21 +1070,40 @@ DEF_CONSOLE_CMD(ConRestart)
 }
 
 #ifdef ENABLE_AI
-DEF_CONSOLE_CMD(ConListAI)
+/**
+ * Print a text buffer line by line to the console. Lines are seperated by '\n'.
+ * @param buf The buffer to print.
+ * @note All newlines are replace by '\0' characters.
+ */
+static void PrintLineByLine(char *buf)
 {
-	char buf[4096];
-	char *p = &buf[0];
-	p = AI::GetConsoleList(p, lastof(buf));
-
-	p = &buf[0];
+	char *p = buf;
 	/* Print output line by line */
-	for (char *p2 = &buf[0]; *p2 != '\0'; p2++) {
+	for (char *p2 = buf; *p2 != '\0'; p2++) {
 		if (*p2 == '\n') {
 			*p2 = '\0';
 			IConsolePrintF(CC_DEFAULT, "%s", p);
 			p = p2 + 1;
 		}
 	}
+}
+
+DEF_CONSOLE_CMD(ConListAILibs)
+{
+	char buf[4096];
+	AI::GetConsoleLibraryList(buf, lastof(buf));
+
+	PrintLineByLine(buf);
+
+	return true;
+}
+
+DEF_CONSOLE_CMD(ConListAI)
+{
+	char buf[4096];
+	AI::GetConsoleList(buf, lastof(buf));
+
+	PrintLineByLine(buf);
 
 	return true;
 }
@@ -1052,7 +1161,7 @@ DEF_CONSOLE_CMD(ConStartAI)
 	}
 
 	/* Start a new AI company */
-	DoCommandP(0, 1, INVALID_COMPANY, CMD_COMPANY_CTRL);
+	DoCommandP(0, 1 | INVALID_COMPANY << 16, 0, CMD_COMPANY_CTRL);
 
 	return true;
 }
@@ -1087,8 +1196,8 @@ DEF_CONSOLE_CMD(ConReloadAI)
 	}
 
 	/* First kill the company of the AI, then start a new one. This should start the current AI again */
-	DoCommandP(0, 2, company_id, CMD_COMPANY_CTRL);
-	DoCommandP(0, 1, company_id, CMD_COMPANY_CTRL);
+	DoCommandP(0, 2 | company_id << 16, 0, CMD_COMPANY_CTRL);
+	DoCommandP(0, 1 | company_id << 16, 0, CMD_COMPANY_CTRL);
 	IConsolePrint(CC_DEFAULT, "AI reloaded.");
 
 	return true;
@@ -1124,7 +1233,7 @@ DEF_CONSOLE_CMD(ConStopAI)
 	}
 
 	/* Now kill the company of the AI. */
-	DoCommandP(0, 2, company_id, CMD_COMPANY_CTRL);
+	DoCommandP(0, 2 | company_id << 16, 0, CMD_COMPANY_CTRL);
 	IConsolePrint(CC_DEFAULT, "AI stopped, company deleted.");
 
 	return true;
@@ -1142,12 +1251,29 @@ DEF_CONSOLE_CMD(ConRescanAI)
 		return true;
 	}
 
+	TarScanner::DoScan();
 	AI::Rescan();
+	InvalidateWindowData(WC_AI_LIST, 0, 1);
+	SetWindowDirty(WC_AI_SETTINGS, 0);
 
 	return true;
 }
 #endif /* ENABLE_AI */
 
+DEF_CONSOLE_CMD(ConRescanNewGRF)
+{
+	if (argc == 0) {
+		IConsoleHelp("Rescan the data dir for NewGRFs. Usage: 'rescan_newgrf'");
+		return true;
+	}
+
+	TarScanner::DoScan();
+	ScanNewGRFFiles();
+	InvalidateWindowData(WC_GAME_OPTIONS, 0, GOID_NEWGRF_RESCANNED);
+
+	return true;
+}
+
 DEF_CONSOLE_CMD(ConGetSeed)
 {
 	if (argc == 0) {
@@ -1198,9 +1324,10 @@ DEF_CONSOLE_CMD(ConAlias)
 DEF_CONSOLE_CMD(ConScreenShot)
 {
 	if (argc == 0) {
-		IConsoleHelp("Create a screenshot of the game. Usage: 'screenshot [big | no_con] [file name]'");
-		IConsoleHelp("'big' makes a screenshot of the whole map, 'no_con' hides the console to create "
-				"the screenshot. Screenshots of whole map are always drawn without console");
+		IConsoleHelp("Create a screenshot of the game. Usage: 'screenshot [big | giant | no_con] [file name]'");
+		IConsoleHelp("'big' makes a zoomed-in screenshot of the visible area, 'giant' makes a screenshot of the "
+				"whole map, 'no_con' hides the console to create the screenshot. 'big' or 'giant' "
+				"screenshots are always drawn without console");
 		return true;
 	}
 
@@ -1212,6 +1339,10 @@ DEF_CONSOLE_CMD(ConScreenShot)
 	if (argc > 1) {
 		if (strcmp(argv[1], "big") == 0) {
 			/* screenshot big [filename] */
+			type = SC_ZOOMEDIN;
+			if (argc > 2) name = argv[2];
+		} else if (strcmp(argv[1], "giant") == 0) {
+			/* screenshot giant [filename] */
 			type = SC_WORLD;
 			if (argc > 2) name = argv[2];
 		} else if (strcmp(argv[1], "no_con") == 0) {
@@ -1233,8 +1364,6 @@ DEF_CONSOLE_CMD(ConScreenShot)
 
 DEF_CONSOLE_CMD(ConInfoCmd)
 {
-	const IConsoleCmd *cmd;
-
 	if (argc == 0) {
 		IConsoleHelp("Print out debugging information about a command. Usage: 'info_cmd <cmd>'");
 		return true;
@@ -1242,7 +1371,7 @@ DEF_CONSOLE_CMD(ConInfoCmd)
 
 	if (argc < 2) return false;
 
-	cmd = IConsoleCmdGet(argv[1]);
+	const IConsoleCmd *cmd = IConsoleCmdGet(argv[1]);
 	if (cmd == NULL) {
 		IConsoleError("the given command was not found");
 		return true;
@@ -1307,6 +1436,7 @@ DEF_CONSOLE_CMD(ConHelp)
 		const IConsoleCmd *cmd;
 		const IConsoleAlias *alias;
 
+		RemoveUnderscores(argv[1]);
 		cmd = IConsoleCmdGet(argv[1]);
 		if (cmd != NULL) {
 			cmd->proc(0, NULL);
@@ -1342,16 +1472,14 @@ DEF_CONSOLE_CMD(ConHelp)
 
 DEF_CONSOLE_CMD(ConListCommands)
 {
-	const IConsoleCmd *cmd;
-
 	if (argc == 0) {
 		IConsoleHelp("List all registered commands. Usage: 'list_cmds [<pre-filter>]'");
 		return true;
 	}
 
-	for (cmd = _iconsole_cmds; cmd != NULL; cmd = cmd->next) {
+	for (const IConsoleCmd *cmd = _iconsole_cmds; cmd != NULL; cmd = cmd->next) {
 		if (argv[1] == NULL || strstr(cmd->name, argv[1]) != NULL) {
-				IConsolePrintF(CC_DEFAULT, "%s", cmd->name);
+			if (cmd->hook == NULL || cmd->hook(false) != CHR_HIDE) IConsolePrintF(CC_DEFAULT, "%s", cmd->name);
 		}
 	}
 
@@ -1360,14 +1488,12 @@ DEF_CONSOLE_CMD(ConListCommands)
 
 DEF_CONSOLE_CMD(ConListAliases)
 {
-	const IConsoleAlias *alias;
-
 	if (argc == 0) {
 		IConsoleHelp("List all registered aliases. Usage: 'list_aliases [<pre-filter>]'");
 		return true;
 	}
 
-	for (alias = _iconsole_aliases; alias != NULL; alias = alias->next) {
+	for (const IConsoleAlias *alias = _iconsole_aliases; alias != NULL; alias = alias->next) {
 		if (argv[1] == NULL || strstr(alias->name, argv[1]) != NULL) {
 			IConsolePrintF(CC_DEFAULT, "%s => %s", alias->name, alias->cmdline);
 		}
@@ -1390,7 +1516,8 @@ DEF_CONSOLE_CMD(ConSay)
 	if (!_network_server) {
 		NetworkClientSendChat(NETWORK_ACTION_CHAT, DESTTYPE_BROADCAST, 0 /* param does not matter */, argv[1]);
 	} else {
-		NetworkServerSendChat(NETWORK_ACTION_CHAT, DESTTYPE_BROADCAST, 0, argv[1], CLIENT_ID_SERVER);
+		bool from_admin = (_redirect_console_to_admin < INVALID_ADMIN_ID);
+		NetworkServerSendChat(NETWORK_ACTION_CHAT, DESTTYPE_BROADCAST, 0, argv[1], CLIENT_ID_SERVER, from_admin);
 	}
 
 	return true;
@@ -1398,8 +1525,6 @@ DEF_CONSOLE_CMD(ConSay)
 
 DEF_CONSOLE_CMD(ConCompanies)
 {
-	Company *c;
-
 	if (argc == 0) {
 		IConsoleHelp("List the in-game details of all clients connected to the server. Usage 'companies'");
 		return true;
@@ -1407,6 +1532,7 @@ DEF_CONSOLE_CMD(ConCompanies)
 	NetworkCompanyStats company_stats[MAX_COMPANIES];
 	NetworkPopulateCompanyStats(company_stats);
 
+	Company *c;
 	FOR_ALL_COMPANIES(c) {
 		/* Grab the company name */
 		char company_name[NETWORK_COMPANY_NAME_LENGTH];
@@ -1448,7 +1574,8 @@ DEF_CONSOLE_CMD(ConSayCompany)
 	if (!_network_server) {
 		NetworkClientSendChat(NETWORK_ACTION_CHAT_COMPANY, DESTTYPE_TEAM, company_id, argv[2]);
 	} else {
-		NetworkServerSendChat(NETWORK_ACTION_CHAT_COMPANY, DESTTYPE_TEAM, company_id, argv[2], CLIENT_ID_SERVER);
+		bool from_admin = (_redirect_console_to_admin < INVALID_ADMIN_ID);
+		NetworkServerSendChat(NETWORK_ACTION_CHAT_COMPANY, DESTTYPE_TEAM, company_id, argv[2], CLIENT_ID_SERVER, from_admin);
 	}
 
 	return true;
@@ -1467,30 +1594,53 @@ DEF_CONSOLE_CMD(ConSayClient)
 	if (!_network_server) {
 		NetworkClientSendChat(NETWORK_ACTION_CHAT_CLIENT, DESTTYPE_CLIENT, atoi(argv[1]), argv[2]);
 	} else {
-		NetworkServerSendChat(NETWORK_ACTION_CHAT_CLIENT, DESTTYPE_CLIENT, atoi(argv[1]), argv[2], CLIENT_ID_SERVER);
+		bool from_admin = (_redirect_console_to_admin < INVALID_ADMIN_ID);
+		NetworkServerSendChat(NETWORK_ACTION_CHAT_CLIENT, DESTTYPE_CLIENT, atoi(argv[1]), argv[2], CLIENT_ID_SERVER, from_admin);
 	}
 
 	return true;
 }
 
-extern void HashCurrentCompanyPassword(const char *password);
-
 DEF_CONSOLE_CMD(ConCompanyPassword)
 {
 	if (argc == 0) {
-		IConsoleHelp("Change the password of your company. Usage: 'company_pw \"<password>\"'");
+		const char *helpmsg;
+
+		if (_network_dedicated) {
+			helpmsg = "Change the password of a company. Usage: 'company_pw <company-no> \"<password>\"";
+		} else if (_network_server) {
+			helpmsg = "Change the password of your or any other company. Usage: 'company_pw [<company-no>] \"<password>\"'";
+		} else {
+			helpmsg = "Change the password of your company. Usage: 'company_pw \"<password>\"'";
+		}
+
+		IConsoleHelp(helpmsg);
 		IConsoleHelp("Use \"*\" to disable the password.");
 		return true;
 	}
 
-	if (argc != 2) return false;
+	CompanyID company_id;
+	const char *password;
+	const char *errormsg;
 
-	if (!Company::IsValidID(_local_company)) {
-		IConsoleError("You have to own a company to make use of this command.");
+	if (argc == 2) {
+		company_id = _local_company;
+		password = argv[1];
+		errormsg = "You have to own a company to make use of this command.";
+	} else if (argc == 3 && _network_server) {
+		company_id = (CompanyID)(atoi(argv[1]) - 1);
+		password = argv[2];
+		errormsg = "You have to specify the ID of a valid human controlled company.";
+	} else {
 		return false;
 	}
 
-	const char *password = NetworkChangeCompanyPassword(argv[1]);
+	if (!Company::IsValidHumanID(company_id)) {
+		IConsoleError(errormsg);
+		return false;
+	}
+
+	password = NetworkChangeCompanyPassword(company_id, password);
 
 	if (StrEmpty(password)) {
 		IConsolePrintF(CC_WARNING, "Company password cleared");
@@ -1594,7 +1744,7 @@ DEF_CONSOLE_CMD(ConContent)
 			static const char * const types[] = { "Base graphics", "NewGRF", "AI", "AI library", "Scenario", "Heightmap", "Base sound", "Base music" };
 			assert_compile(lengthof(types) == CONTENT_TYPE_END - CONTENT_TYPE_BEGIN);
 			static const char * const states[] = { "Not selected", "Selected", "Dep Selected", "Installed", "Unknown" };
-			static const ConsoleColour state_to_colour[] = { CC_COMMAND, CC_INFO, CC_INFO, CC_WHITE, CC_ERROR };
+			static const TextColour state_to_colour[] = { CC_COMMAND, CC_INFO, CC_INFO, CC_WHITE, CC_ERROR };
 
 			const ContentInfo *ci = *iter;
 			IConsolePrintF(state_to_colour[ci->state], "%d, %s, %s, %s", ci->id, types[ci->type - 1], states[ci->state], ci->name);
@@ -1672,6 +1822,17 @@ DEF_CONSOLE_CMD(ConGamelogPrint)
 	return true;
 }
 
+DEF_CONSOLE_CMD(ConNewGRFReload)
+{
+	if (argc == 0) {
+		IConsoleHelp("Reloads all active NewGRFs from disk. Equivalent to reapplying NewGRFs via the settings, but without asking for confirmation. This might crash OpenTTD!");
+		return true;
+	}
+
+	ReloadNewGRFData();
+	return true;
+}
+
 #ifdef _DEBUG
 /******************
  *  debug commands
@@ -1680,7 +1841,6 @@ DEF_CONSOLE_CMD(ConGamelogPrint)
 static void IConsoleDebugLibRegister()
 {
 	IConsoleCmdRegister("resettile",        ConResetTile);
-	IConsoleCmdRegister("stopall",          ConStopAllVehicles);
 	IConsoleAliasRegister("dbg_echo",       "echo %A; echo %B");
 	IConsoleAliasRegister("dbg_echo2",      "echo %!");
 }
@@ -1708,6 +1868,7 @@ void IConsoleStdLibRegister()
 	IConsoleCmdRegister("getdate",      ConGetDate);
 	IConsoleCmdRegister("quit",         ConExit);
 	IConsoleCmdRegister("resetengines", ConResetEngines, ConHookNoNetwork);
+	IConsoleCmdRegister("reset_enginepool", ConResetEnginePool, ConHookNoNetwork);
 	IConsoleCmdRegister("return",       ConReturn);
 	IConsoleCmdRegister("screenshot",   ConScreenShot);
 	IConsoleCmdRegister("script",       ConScript);
@@ -1725,12 +1886,11 @@ void IConsoleStdLibRegister()
 	IConsoleCmdRegister("setting_newgame", ConSettingNewgame);
 	IConsoleCmdRegister("list_settings",ConListSettings);
 	IConsoleCmdRegister("gamelog",      ConGamelogPrint);
+	IConsoleCmdRegister("rescan_newgrf", ConRescanNewGRF);
 
 	IConsoleAliasRegister("dir",          "ls");
 	IConsoleAliasRegister("del",          "rm %+");
 	IConsoleAliasRegister("newmap",       "newgame");
-	IConsoleAliasRegister("new_map",      "newgame");
-	IConsoleAliasRegister("new_game",     "newgame");
 	IConsoleAliasRegister("patch",        "setting %+");
 	IConsoleAliasRegister("set",          "setting %+");
 	IConsoleAliasRegister("set_newgame",  "setting_newgame %+");
@@ -1738,6 +1898,7 @@ void IConsoleStdLibRegister()
 	IConsoleAliasRegister("developer",    "setting developer %+");
 
 #ifdef ENABLE_AI
+	IConsoleCmdRegister("list_ai_libs", ConListAILibs);
 	IConsoleCmdRegister("list_ai",      ConListAI);
 	IConsoleCmdRegister("reload_ai",    ConReloadAI);
 	IConsoleCmdRegister("rescan_ai",    ConRescanAI);
@@ -1812,4 +1973,7 @@ void IConsoleStdLibRegister()
 #ifdef _DEBUG
 	IConsoleDebugLibRegister();
 #endif
+
+	/* NewGRF development stuff */
+	IConsoleCmdRegister("reload_newgrfs",  ConNewGRFReload, ConHookNewGRFDeveloperTool);
 }
