@@ -1,4 +1,4 @@
-/* $Id: network.cpp 20520 2010-08-16 23:06:22Z rubidium $ */
+/* $Id: network.cpp 22935 2011-09-15 18:56:43Z rubidium $ */
 
 /*
  * This file is part of OpenTTD.
@@ -16,6 +16,7 @@
 #include "../strings_func.h"
 #include "../command_func.h"
 #include "../date_func.h"
+#include "network_admin.h"
 #include "network_client.h"
 #include "network_server.h"
 #include "network_content.h"
@@ -43,8 +44,6 @@
 bool _ddc_fastforward = true;
 #endif /* DEBUG_DUMP_COMMANDS */
 
-DECLARE_POSTFIX_INCREMENT(ClientID);
-
 assert_compile(NetworkClientInfoPool::MAX_SIZE == NetworkClientSocketPool::MAX_SIZE);
 
 NetworkClientInfoPool _networkclientinfo_pool("NetworkClientInfo");
@@ -83,19 +82,14 @@ CompanyMask _network_company_passworded; ///< Bitmask of the password status of
 
 /* Check whether NETWORK_NUM_LANDSCAPES is still in sync with NUM_LANDSCAPE */
 assert_compile((int)NETWORK_NUM_LANDSCAPES == (int)NUM_LANDSCAPE);
-assert_compile((int)NETWORK_COMPANY_NAME_LENGTH == MAX_LENGTH_COMPANY_NAME_BYTES);
+assert_compile((int)NETWORK_COMPANY_NAME_LENGTH == MAX_LENGTH_COMPANY_NAME_CHARS * MAX_CHAR_LENGTH);
 
 extern NetworkUDPSocketHandler *_udp_client_socket; ///< udp client socket
 extern NetworkUDPSocketHandler *_udp_server_socket; ///< udp server socket
 extern NetworkUDPSocketHandler *_udp_master_socket; ///< udp master socket
 
-/* The listen socket for the server */
-static SocketList _listensockets;
-
 /* The amount of clients connected */
-static byte _network_clients_connected = 0;
-/* The identifier counter for new clients (is never decreased) */
-static ClientID _network_client_id = CLIENT_ID_FIRST;
+byte _network_clients_connected = 0;
 
 /* Some externs / forwards */
 extern void StateGameLoop();
@@ -109,22 +103,12 @@ NetworkClientInfo::~NetworkClientInfo()
 	InvalidateWindowData(WC_SEND_NETWORK_MSG, DESTTYPE_CLIENT, this->client_id);
 }
 
-/**
- * Return the CI given it's raw index
- * @param index the index to search for
- * @return return a pointer to the corresponding NetworkClientInfo struct
- */
-NetworkClientInfo *NetworkFindClientInfoFromIndex(ClientIndex index)
-{
-	return NetworkClientInfo::GetIfValid(index);
-}
-
 /**
  * Return the CI given it's client-identifier
  * @param client_id the ClientID to search for
  * @return return a pointer to the corresponding NetworkClientInfo struct or NULL when not found
  */
-NetworkClientInfo *NetworkFindClientInfoFromClientID(ClientID client_id)
+/* static */ NetworkClientInfo *NetworkClientInfo::GetByClientID(ClientID client_id)
 {
 	NetworkClientInfo *ci;
 
@@ -135,31 +119,12 @@ NetworkClientInfo *NetworkFindClientInfoFromClientID(ClientID client_id)
 	return NULL;
 }
 
-/**
- * Return the CI for a given IP
- * @param ip IP of the client we are looking for. This must be in string-format
- * @return return a pointer to the corresponding NetworkClientInfo struct or NULL when not found
- */
-NetworkClientInfo *NetworkFindClientInfoFromIP(const char *ip)
-{
-	NetworkClientInfo *ci;
-	NetworkAddress address(ip);
-
-	if (address.GetAddressLength() == 0) return NULL;
-
-	FOR_ALL_CLIENT_INFOS(ci) {
-		if (ci->client_address == address) return ci;
-	}
-
-	return NULL;
-}
-
 /**
  * Return the client state given it's client-identifier
  * @param client_id the ClientID to search for
  * @return return a pointer to the corresponding NetworkClientSocket struct or NULL when not found
  */
-NetworkClientSocket *NetworkFindClientStateFromClientID(ClientID client_id)
+/* static */ NetworkClientSocket *NetworkClientSocket::GetByClientID(ClientID client_id)
 {
 	NetworkClientSocket *cs;
 
@@ -170,19 +135,6 @@ NetworkClientSocket *NetworkFindClientStateFromClientID(ClientID client_id)
 	return NULL;
 }
 
-/* NetworkGetClientName is a server-safe function to get the name of the client
- *  if the user did not send it yet, Client #<no> is used. */
-void NetworkGetClientName(char *client_name, size_t size, const NetworkClientSocket *cs)
-{
-	const NetworkClientInfo *ci = cs->GetInfo();
-
-	if (StrEmpty(ci->client_name)) {
-		snprintf(client_name, size, "Client #%4d", cs->client_id);
-	} else {
-		ttd_strlcpy(client_name, ci->client_name, size);
-	}
-}
-
 byte NetworkSpectatorCount()
 {
 	const NetworkClientInfo *ci;
@@ -198,6 +150,59 @@ byte NetworkSpectatorCount()
 	return count;
 }
 
+/**
+ * Change the company password of a given company.
+ * @param company_id ID of the company the password should be changed for.
+ * @param password The unhashed password we like to set ('*' or '' resets the password)
+ * @return The password.
+ */
+const char *NetworkChangeCompanyPassword(CompanyID company_id, const char *password)
+{
+	if (strcmp(password, "*") == 0) password = "";
+
+	if (_network_server) {
+		NetworkServerSetCompanyPassword(company_id, password, false);
+	} else {
+		NetworkClientSetCompanyPassword(password);
+	}
+
+	return password;
+}
+
+/**
+ * Hash the given password using server ID and game seed.
+ * @param password Password to hash.
+ * @param password_server_id Server ID.
+ * @param password_game_seed Game seed.
+ * @return The hashed password.
+ */
+const char *GenerateCompanyPasswordHash(const char *password, const char *password_server_id, uint32 password_game_seed)
+{
+	if (StrEmpty(password)) return password;
+
+	char salted_password[NETWORK_SERVER_ID_LENGTH];
+
+	memset(salted_password, 0, sizeof(salted_password));
+	snprintf(salted_password, sizeof(salted_password), "%s", password);
+	/* Add the game seed and the server's ID as the salt. */
+	for (uint i = 0; i < NETWORK_SERVER_ID_LENGTH - 1; i++) {
+		salted_password[i] ^= password_server_id[i] ^ (password_game_seed >> (i % 32));
+	}
+
+	Md5 checksum;
+	uint8 digest[16];
+	static char hashed_password[NETWORK_SERVER_ID_LENGTH];
+
+	/* Generate the MD5 hash */
+	checksum.Append(salted_password, sizeof(salted_password) - 1);
+	checksum.Finish(digest);
+
+	for (int di = 0; di < 16; di++) sprintf(hashed_password + di * 2, "%02x", digest[di]);
+	hashed_password[lengthof(hashed_password) - 1] = '\0';
+
+	return hashed_password;
+}
+
 /**
  * Check if the company we want to join requires a password.
  * @param company_id id of the company we want to check the 'passworded' flag for.
@@ -211,10 +216,8 @@ bool NetworkCompanyIsPassworded(CompanyID company_id)
 /* This puts a text-message to the console, or in the future, the chat-box,
  *  (to keep it all a bit more general)
  * If 'self_send' is true, this is the client who is sending the message */
-void NetworkTextMessage(NetworkAction action, ConsoleColour colour, bool self_send, const char *name, const char *str, int64 data)
+void NetworkTextMessage(NetworkAction action, TextColour colour, bool self_send, const char *name, const char *str, int64 data)
 {
-	const int duration = 10; // Game days the messages stay visible
-
 	StringID strid;
 	switch (action) {
 		case NETWORK_ACTION_SERVER_MESSAGE:
@@ -250,11 +253,17 @@ void NetworkTextMessage(NetworkAction action, ConsoleColour colour, bool self_se
 	SetDParamStr(0, name);
 	SetDParamStr(1, str);
 	SetDParam(2, data);
-	GetString(message, strid, lastof(message));
+
+	/* All of these strings start with "***". These characters are interpreted as both left-to-right and
+	 * right-to-left characters depending on the context. As the next text might be an user's name, the
+	 * user name's characters will influence the direction of the "***" instead of the language setting
+	 * of the game. Manually set the direction of the "***" by inserting a text-direction marker. */
+	char *msg_ptr = message + Utf8Encode(message, _current_text_dir == TD_LTR ? CHAR_TD_LRM : CHAR_TD_RLM);
+	GetString(msg_ptr, strid, lastof(message));
 
 	DEBUG(desync, 1, "msg: %08x; %02x; %s", _date, _date_fract, message);
 	IConsolePrintF(colour, "%s", message);
-	NetworkAddChatMessage((TextColour)colour, duration, "%s", message);
+	NetworkAddChatMessage((TextColour)colour, _settings_client.gui.network_chat_timeout, "%s", message);
 }
 
 /* Calculate the frame-lag of a client */
@@ -264,62 +273,22 @@ uint NetworkCalculateLag(const NetworkClientSocket *cs)
 	/* This client has missed his ACK packet after 1 DAY_TICKS..
 	 *  so we increase his lag for every frame that passes!
 	 * The packet can be out by a max of _net_frame_freq */
-	if (cs->last_frame_server + DAY_TICKS + _settings_client.network.frame_freq < _frame_counter)
+	if (cs->last_frame_server + DAY_TICKS + _settings_client.network.frame_freq < _frame_counter) {
 		lag += _frame_counter - (cs->last_frame_server + DAY_TICKS + _settings_client.network.frame_freq);
-
+	}
 	return lag;
 }
 
 
 /* There was a non-recoverable error, drop back to the main menu with a nice
  *  error */
-static void NetworkError(StringID error_string)
+void NetworkError(StringID error_string)
 {
 	_switch_mode = SM_MENU;
 	extern StringID _switch_mode_errorstr;
 	_switch_mode_errorstr = error_string;
 }
 
-static void ServerStartError(const char *error)
-{
-	DEBUG(net, 0, "[server] could not start network: %s",error);
-	NetworkError(STR_NETWORK_ERROR_SERVER_START);
-}
-
-static void NetworkClientError(NetworkRecvStatus res, NetworkClientSocket *cs)
-{
-	/* First, send a CLIENT_ERROR to the server, so he knows we are
-	 *  disconnection (and why!) */
-	NetworkErrorCode errorno;
-
-	/* We just want to close the connection.. */
-	if (res == NETWORK_RECV_STATUS_CLOSE_QUERY) {
-		cs->NetworkSocketHandler::CloseConnection();
-		NetworkCloseClient(cs, res);
-		_networking = false;
-
-		DeleteWindowById(WC_NETWORK_STATUS_WINDOW, 0);
-		return;
-	}
-
-	switch (res) {
-		case NETWORK_RECV_STATUS_DESYNC:          errorno = NETWORK_ERROR_DESYNC; break;
-		case NETWORK_RECV_STATUS_SAVEGAME:        errorno = NETWORK_ERROR_SAVEGAME_FAILED; break;
-		case NETWORK_RECV_STATUS_NEWGRF_MISMATCH: errorno = NETWORK_ERROR_NEWGRF_MISMATCH; break;
-		default:                                  errorno = NETWORK_ERROR_GENERAL; break;
-	}
-
-	/* This means we fucked up and the server closed the connection */
-	if (res != NETWORK_RECV_STATUS_SERVER_ERROR && res != NETWORK_RECV_STATUS_SERVER_FULL &&
-			res != NETWORK_RECV_STATUS_SERVER_BANNED) {
-		SEND_COMMAND(PACKET_CLIENT_ERROR)(errorno);
-	}
-
-	_switch_mode = SM_MENU;
-	NetworkCloseClient(cs, res);
-	_networking = false;
-}
-
 /**
  * Retrieve the string id of an internal error number
  * @param err NetworkErrorCode
@@ -344,7 +313,8 @@ StringID GetNetworkErrorMsg(NetworkErrorCode err)
 		STR_NETWORK_ERROR_CLIENT_COMPANY_MISMATCH,
 		STR_NETWORK_ERROR_CLIENT_KICKED,
 		STR_NETWORK_ERROR_CLIENT_CHEATER,
-		STR_NETWORK_ERROR_CLIENT_SERVER_FULL
+		STR_NETWORK_ERROR_CLIENT_SERVER_FULL,
+		STR_NETWORK_ERROR_CLIENT_TOO_MANY_COMMANDS
 	};
 
 	if (err >= (ptrdiff_t)lengthof(network_error_strings)) err = NETWORK_ERROR_GENERAL;
@@ -389,7 +359,8 @@ void NetworkHandlePauseChange(PauseMode prev_mode, PauseMode changed_mode)
 			char buffer[DRAW_STRING_BUFFER];
 			GetString(buffer, str, lastof(buffer));
 			NetworkTextMessage(NETWORK_ACTION_SERVER_MESSAGE, CC_DEFAULT, false, NULL, buffer);
-		} break;
+			break;
+		}
 
 		default:
 			return;
@@ -423,7 +394,7 @@ static uint NetworkCountActiveClients()
 	uint count = 0;
 
 	FOR_ALL_CLIENT_SOCKETS(cs) {
-		if (cs->status != STATUS_ACTIVE) continue;
+		if (cs->status != NetworkClientSocket::STATUS_ACTIVE) continue;
 		if (!Company::IsValidID(cs->GetInfo()->client_playas)) continue;
 		count++;
 	}
@@ -452,7 +423,7 @@ static bool NetworkHasJoiningClient()
 {
 	const NetworkClientSocket *cs;
 	FOR_ALL_CLIENT_SOCKETS(cs) {
-		if (cs->status >= STATUS_AUTHORIZED && cs->status < STATUS_ACTIVE) return true;
+		if (cs->status >= NetworkClientSocket::STATUS_AUTHORIZED && cs->status < NetworkClientSocket::STATUS_ACTIVE) return true;
 	}
 
 	return false;
@@ -470,12 +441,14 @@ static void CheckPauseOnJoin()
 	CheckPauseHelper(NetworkHasJoiningClient(), PM_PAUSED_JOIN);
 }
 
-/** Converts a string to ip/port/company
+/**
+ * Converts a string to ip/port/company
  *  Format: IP:port#company
  *
  * connection_string will be re-terminated to seperate out the hostname, and company and port will
  * be set to the company and port strings given by the user, inside the memory area originally
- * occupied by connection_string. */
+ * occupied by connection_string.
+ */
 void ParseConnectionString(const char **company, const char **port, char *connection_string)
 {
 	bool ipv6 = (strchr(connection_string, ':') != strrchr(connection_string, ':'));
@@ -504,189 +477,50 @@ void ParseConnectionString(const char **company, const char **port, char *connec
 	}
 }
 
-/* Creates a new client from a socket
- *   Used both by the server and the client */
-static NetworkClientSocket *NetworkAllocClient(SOCKET s)
+/* static */ void ServerNetworkGameSocketHandler::AcceptConnection(SOCKET s, const NetworkAddress &address)
 {
-	if (_network_server) {
-		/* Can we handle a new client? */
-		if (_network_clients_connected >= MAX_CLIENTS) return NULL;
-		if (_network_game_info.clients_on >= _settings_client.network.max_clients) return NULL;
+	/* Register the login */
+	_network_clients_connected++;
 
-		/* Register the login */
-		_network_clients_connected++;
-	}
-
-	NetworkClientSocket *cs = new NetworkClientSocket(INVALID_CLIENT_ID);
-	cs->sock = s;
-	cs->last_frame = _frame_counter;
-	cs->last_frame_server = _frame_counter;
-
-	if (_network_server) {
-		cs->client_id = _network_client_id++;
-		NetworkClientInfo *ci = new NetworkClientInfo(cs->client_id);
-		cs->SetInfo(ci);
-		ci->client_playas = COMPANY_INACTIVE_CLIENT;
-		ci->join_date = _date;
-
-		SetWindowDirty(WC_CLIENT_LIST, 0);
-	}
-
-	return cs;
+	SetWindowDirty(WC_CLIENT_LIST, 0);
+	ServerNetworkGameSocketHandler *cs = new ServerNetworkGameSocketHandler(s);
+	cs->client_address = address; // Save the IP of the client
 }
 
-/* Close a connection */
-NetworkRecvStatus NetworkCloseClient(NetworkClientSocket *cs, NetworkRecvStatus status)
-{
-	assert(status != NETWORK_RECV_STATUS_OKAY);
-	/*
-	 * Sending a message just before leaving the game calls cs->Send_Packets.
-	 * This might invoke this function, which means that when we close the
-	 * connection after cs->Send_Packets we will close an already closed
-	 * connection. This handles that case gracefully without having to make
-	 * that code any more complex or more aware of the validity of the socket.
-	 */
-	if (cs->sock == INVALID_SOCKET) return status;
-
-	if (status != NETWORK_RECV_STATUS_CONN_LOST && !cs->HasClientQuit() && _network_server && cs->status >= STATUS_AUTHORIZED) {
-		/* We did not receive a leave message from this client... */
-		char client_name[NETWORK_CLIENT_NAME_LENGTH];
-		NetworkClientSocket *new_cs;
-
-		NetworkGetClientName(client_name, sizeof(client_name), cs);
-
-		NetworkTextMessage(NETWORK_ACTION_LEAVE, CC_DEFAULT, false, client_name, NULL, STR_NETWORK_ERROR_CLIENT_CONNECTION_LOST);
-
-		/* Inform other clients of this... strange leaving ;) */
-		FOR_ALL_CLIENT_SOCKETS(new_cs) {
-			if (new_cs->status > STATUS_AUTHORIZED && cs != new_cs) {
-				SEND_COMMAND(PACKET_SERVER_ERROR_QUIT)(new_cs, cs->client_id, NETWORK_ERROR_CONNECTION_LOST);
-			}
-		}
-	}
-
-	DEBUG(net, 1, "Closed client connection %d", cs->client_id);
-
-	if (_network_server) {
-		/* We just lost one client :( */
-		if (cs->status >= STATUS_AUTHORIZED) _network_game_info.clients_on--;
-		_network_clients_connected--;
-
-		SetWindowDirty(WC_CLIENT_LIST, 0);
-	}
-
-	cs->Send_Packets(true);
-
-	delete cs->GetInfo();
-	delete cs;
-
-	return status;
-}
-
-/* For the server, to accept new clients */
-static void NetworkAcceptClients(SOCKET ls)
-{
-	for (;;) {
-		struct sockaddr_storage sin;
-		memset(&sin, 0, sizeof(sin));
-		socklen_t sin_len = sizeof(sin);
-		SOCKET s = accept(ls, (struct sockaddr*)&sin, &sin_len);
-		if (s == INVALID_SOCKET) return;
-
-		SetNonBlocking(s); // XXX error handling?
-
-		NetworkAddress address(sin, sin_len);
-		DEBUG(net, 1, "Client connected from %s on frame %d", address.GetHostname(), _frame_counter);
-
-		SetNoDelay(s); // XXX error handling?
-
-		/* Check if the client is banned */
-		bool banned = false;
-		for (char **iter = _network_ban_list.Begin(); iter != _network_ban_list.End(); iter++) {
-			banned = address.IsInNetmask(*iter);
-			if (banned) {
-				Packet p(PACKET_SERVER_BANNED);
-				p.PrepareToSend();
-
-				DEBUG(net, 1, "Banned ip tried to join (%s), refused", *iter);
-
-				send(s, (const char*)p.buffer, p.size, 0);
-				closesocket(s);
-				break;
-			}
-		}
-		/* If this client is banned, continue with next client */
-		if (banned) continue;
-
-		NetworkClientSocket *cs = NetworkAllocClient(s);
-		if (cs == NULL) {
-			/* no more clients allowed?
-			 * Send to the client that we are full! */
-			Packet p(PACKET_SERVER_FULL);
-			p.PrepareToSend();
-
-			send(s, (const char*)p.buffer, p.size, 0);
-			closesocket(s);
-
-			continue;
-		}
-
-		/* a new client has connected. We set him at inactive for now
-		 *  maybe he is only requesting server-info. Till he has sent a PACKET_CLIENT_MAP_OK
-		 *  the client stays inactive */
-		cs->status = STATUS_INACTIVE;
-
-		cs->GetInfo()->client_address = address; // Save the IP of the client
-	}
-}
-
-/* Set up the listen socket for the server */
-static bool NetworkListen()
-{
-	assert(_listensockets.Length() == 0);
-
-	NetworkAddressList addresses;
-	GetBindAddresses(&addresses, _settings_client.network.server_port);
-
-	for (NetworkAddress *address = addresses.Begin(); address != addresses.End(); address++) {
-		address->Listen(SOCK_STREAM, &_listensockets);
-	}
-
-	if (_listensockets.Length() == 0) {
-		ServerStartError("Could not create listening socket");
-		return false;
-	}
-
-	return true;
-}
-
-/** Resets both pools used for network clients */
-static void InitializeNetworkPools()
+/**
+ * Resets the pools used for network clients, and the admin pool if needed.
+ * @param close_admins Whether the admin pool has to be cleared as well.
+ */
+static void InitializeNetworkPools(bool close_admins = true)
 {
 	_networkclientsocket_pool.CleanPool();
 	_networkclientinfo_pool.CleanPool();
+	if (close_admins) _networkadminsocket_pool.CleanPool();
 }
 
-/* Close all current connections */
-static void NetworkClose()
+/**
+ * Close current connections.
+ * @param close_admins Whether the admin connections have to be closed as well.
+ */
+void NetworkClose(bool close_admins)
 {
-	NetworkClientSocket *cs;
-
-	FOR_ALL_CLIENT_SOCKETS(cs) {
-		if (!_network_server) {
-			SEND_COMMAND(PACKET_CLIENT_QUIT)();
-			cs->Send_Packets();
-		}
-		NetworkCloseClient(cs, NETWORK_RECV_STATUS_CONN_LOST);
-	}
-
 	if (_network_server) {
-		/* We are a server, also close the listensocket */
-		for (SocketList::iterator s = _listensockets.Begin(); s != _listensockets.End(); s++) {
-			closesocket(s->second);
+		if (close_admins) {
+			ServerNetworkAdminSocketHandler *as;
+			FOR_ALL_ADMIN_SOCKETS(as) {
+				as->CloseConnection(true);
+			}
 		}
-		_listensockets.Clear();
-		DEBUG(net, 1, "[tcp] closed listeners");
+
+		NetworkClientSocket *cs;
+		FOR_ALL_CLIENT_SOCKETS(cs) {
+			cs->CloseConnection(NETWORK_RECV_STATUS_CONN_LOST);
+		}
+		ServerNetworkGameSocketHandler::CloseListeners();
+		ServerNetworkAdminSocketHandler::CloseListeners();
+	} else if (MyClient::my_client != NULL) {
+		MyClient::SendQuit();
+		MyClient::my_client->CloseConnection(NETWORK_RECV_STATUS_CONN_LOST);
 	}
 
 	TCPConnecter::KillAll();
@@ -699,13 +533,13 @@ static void NetworkClose()
 	free(_network_company_states);
 	_network_company_states = NULL;
 
-	InitializeNetworkPools();
+	InitializeNetworkPools(close_admins);
 }
 
 /* Inits the network (cleans sockets and stuff) */
-static void NetworkInitialize()
+static void NetworkInitialize(bool close_admins = true)
 {
-	InitializeNetworkPools();
+	InitializeNetworkPools(close_admins);
 	NetworkUDPInitialize();
 
 	_sync_frame = 0;
@@ -727,8 +561,8 @@ public:
 	virtual void OnConnect(SOCKET s)
 	{
 		_networking = true;
-		NetworkAllocClient(s);
-		SEND_COMMAND(PACKET_CLIENT_COMPANY_INFO)();
+		new ClientNetworkGameSocketHandler(s);
+		MyClient::SendCompanyInformationQuery();
 	}
 };
 
@@ -810,7 +644,7 @@ public:
 	virtual void OnConnect(SOCKET s)
 	{
 		_networking = true;
-		NetworkAllocClient(s);
+		new ClientNetworkGameSocketHandler(s);
 		IConsoleCmdExec("exec scripts/on_client.scr 0");
 		NetworkClient_Connected();
 	}
@@ -848,13 +682,10 @@ static void NetworkInitGameInfo()
 	/* The server is a client too */
 	_network_game_info.clients_on = _network_dedicated ? 0 : 1;
 
+	/* There should be always space for the server. */
+	assert(NetworkClientInfo::CanAllocateItem());
 	NetworkClientInfo *ci = new NetworkClientInfo(CLIENT_ID_SERVER);
 	ci->client_playas = _network_dedicated ? COMPANY_SPECTATOR : _local_company;
-	/* Give the server a valid IP; banning it is pointless anyways */
-	sockaddr_in sock;
-	memset(&sock, 0, sizeof(sock));
-	sock.sin_family = AF_INET;
-	ci->client_address = NetworkAddress((sockaddr*)&sock, sizeof(sock));
 
 	strecpy(ci->client_name, _settings_client.network.client_name, lastof(ci->client_name));
 }
@@ -867,9 +698,12 @@ bool NetworkServerStart()
 	IConsoleCmdExec("exec scripts/pre_server.scr 0");
 	if (_network_dedicated) IConsoleCmdExec("exec scripts/pre_dedicated.scr 0");
 
-	NetworkDisconnect();
-	NetworkInitialize();
-	if (!NetworkListen()) return false;
+	NetworkDisconnect(false, false);
+	NetworkInitialize(false);
+	if (!ServerNetworkGameSocketHandler::Listen(_settings_client.network.server_port)) return false;
+
+	/* Only listen for admins when the password isn't empty. */
+	if (!StrEmpty(_settings_client.network.admin_password) && !ServerNetworkAdminSocketHandler::Listen(_settings_client.network.server_admin_port)) return false;
 
 	/* Try to start UDP-server */
 	_network_udp_server = _udp_server_socket->Listen();
@@ -897,6 +731,10 @@ bool NetworkServerStart()
 	_network_last_advertise_frame = 0;
 	_network_need_advertise = true;
 	NetworkUDPAdvertise();
+
+	/* welcome possibly still connected admins - this can only happen on a dedicated server. */
+	if (_network_dedicated) ServerNetworkAdminSocketHandler::WelcomeAll();
+
 	return true;
 }
 
@@ -907,25 +745,42 @@ void NetworkReboot()
 	if (_network_server) {
 		NetworkClientSocket *cs;
 		FOR_ALL_CLIENT_SOCKETS(cs) {
-			SEND_COMMAND(PACKET_SERVER_NEWGAME)(cs);
-			cs->Send_Packets();
+			cs->SendNewGame();
+			cs->SendPackets();
+		}
+
+		ServerNetworkAdminSocketHandler *as;
+		FOR_ALL_ACTIVE_ADMIN_SOCKETS(as) {
+			as->SendNewGame();
+			as->SendPackets();
 		}
 	}
 
-	NetworkClose();
+	/* For non-dedicated servers we have to kick the admins as we are not
+	 * certain that we will end up in a new network game. */
+	NetworkClose(!_network_dedicated);
 }
 
 /**
  * We want to disconnect from the host/clients.
- * @param blocking whether to wait till everything has been closed
+ * @param blocking whether to wait till everything has been closed.
+ * @param close_admins Whether the admin sockets need to be closed as well.
  */
-void NetworkDisconnect(bool blocking)
+void NetworkDisconnect(bool blocking, bool close_admins)
 {
 	if (_network_server) {
 		NetworkClientSocket *cs;
 		FOR_ALL_CLIENT_SOCKETS(cs) {
-			SEND_COMMAND(PACKET_SERVER_SHUTDOWN)(cs);
-			cs->Send_Packets();
+			cs->SendShutdown();
+			cs->SendPackets();
+		}
+
+		if (close_admins) {
+			ServerNetworkAdminSocketHandler *as;
+			FOR_ALL_ACTIVE_ADMIN_SOCKETS(as) {
+				as->SendShutdown();
+				as->SendPackets();
+			}
 		}
 	}
 
@@ -933,7 +788,7 @@ void NetworkDisconnect(bool blocking)
 
 	DeleteWindowById(WC_NETWORK_STATUS_WINDOW, 0);
 
-	NetworkClose();
+	NetworkClose(close_admins);
 
 	/* Reinitialize the UDP stack, i.e. close all existing connections. */
 	NetworkUDPInitialize();
@@ -945,118 +800,25 @@ void NetworkDisconnect(bool blocking)
  */
 static bool NetworkReceive()
 {
-	NetworkClientSocket *cs;
-	fd_set read_fd, write_fd;
-	struct timeval tv;
-
-	FD_ZERO(&read_fd);
-	FD_ZERO(&write_fd);
-
-	FOR_ALL_CLIENT_SOCKETS(cs) {
-		FD_SET(cs->sock, &read_fd);
-		FD_SET(cs->sock, &write_fd);
+	if (_network_server) {
+		ServerNetworkAdminSocketHandler::Receive();
+		return ServerNetworkGameSocketHandler::Receive();
+	} else {
+		return ClientNetworkGameSocketHandler::Receive();
 	}
-
-	/* take care of listener port */
-	for (SocketList::iterator s = _listensockets.Begin(); s != _listensockets.End(); s++) {
-		FD_SET(s->second, &read_fd);
-	}
-
-	tv.tv_sec = tv.tv_usec = 0; // don't block at all.
-#if !defined(__MORPHOS__) && !defined(__AMIGA__)
-	int n = select(FD_SETSIZE, &read_fd, &write_fd, NULL, &tv);
-#else
-	int n = WaitSelect(FD_SETSIZE, &read_fd, &write_fd, NULL, &tv, NULL);
-#endif
-	if (n == -1 && !_network_server) NetworkError(STR_NETWORK_ERROR_LOSTCONNECTION);
-
-	/* accept clients.. */
-	for (SocketList::iterator s = _listensockets.Begin(); s != _listensockets.End(); s++) {
-		if (FD_ISSET(s->second, &read_fd)) NetworkAcceptClients(s->second);
-	}
-
-	/* read stuff from clients */
-	FOR_ALL_CLIENT_SOCKETS(cs) {
-		cs->writable = !!FD_ISSET(cs->sock, &write_fd);
-		if (FD_ISSET(cs->sock, &read_fd)) {
-			if (_network_server) {
-				NetworkServer_ReadPackets(cs);
-			} else {
-				NetworkRecvStatus res;
-
-				/* The client already was quiting! */
-				if (cs->HasClientQuit()) return false;
-
-				res = NetworkClient_ReadPackets(cs);
-				if (res != NETWORK_RECV_STATUS_OKAY) {
-					/* The client made an error of which we can not recover
-					 *   close the client and drop back to main menu */
-					NetworkClientError(res, cs);
-					return false;
-				}
-			}
-		}
-	}
-	return _networking;
 }
 
 /* This sends all buffered commands (if possible) */
 static void NetworkSend()
 {
-	NetworkClientSocket *cs;
-	FOR_ALL_CLIENT_SOCKETS(cs) {
-		if (cs->writable) {
-			cs->Send_Packets();
-
-			if (cs->status == STATUS_MAP) {
-				/* This client is in the middle of a map-send, call the function for that */
-				SEND_COMMAND(PACKET_SERVER_MAP)(cs);
-			}
-		}
+	if (_network_server) {
+		ServerNetworkAdminSocketHandler::Send();
+		ServerNetworkGameSocketHandler::Send();
+	} else {
+		ClientNetworkGameSocketHandler::Send();
 	}
 }
 
-static bool NetworkDoClientLoop()
-{
-	_frame_counter++;
-
-	NetworkExecuteLocalCommandQueue();
-
-	StateGameLoop();
-
-	/* Check if we are in sync! */
-	if (_sync_frame != 0) {
-		if (_sync_frame == _frame_counter) {
-#ifdef NETWORK_SEND_DOUBLE_SEED
-			if (_sync_seed_1 != _random.state[0] || _sync_seed_2 != _random.state[1]) {
-#else
-			if (_sync_seed_1 != _random.state[0]) {
-#endif
-				NetworkError(STR_NETWORK_ERROR_DESYNC);
-				DEBUG(desync, 1, "sync_err: %08x; %02x", _date, _date_fract);
-				DEBUG(net, 0, "Sync error detected!");
-				NetworkClientError(NETWORK_RECV_STATUS_DESYNC, NetworkClientSocket::Get(0));
-				return false;
-			}
-
-			/* If this is the first time we have a sync-frame, we
-			 *   need to let the server know that we are ready and at the same
-			 *   frame as he is.. so we can start playing! */
-			if (_network_first_time) {
-				_network_first_time = false;
-				SEND_COMMAND(PACKET_CLIENT_ACK)();
-			}
-
-			_sync_frame = 0;
-		} else if (_sync_frame < _frame_counter) {
-			DEBUG(net, 1, "Missed frame for sync-test (%d / %d)", _sync_frame, _frame_counter);
-			_sync_frame = 0;
-		}
-	}
-
-	return true;
-}
-
 /* We have to do some UDP checking */
 void NetworkUDPGameLoop()
 {
@@ -1070,7 +832,6 @@ void NetworkUDPGameLoop()
 	} else {
 		_udp_client_socket->ReceivePackets();
 		if (_network_udp_broadcast > 0) _network_udp_broadcast--;
-		NetworkGameListRequery();
 	}
 }
 
@@ -1109,7 +870,7 @@ void NetworkGameLoop()
 		while (f != NULL && !feof(f)) {
 			if (_date == next_date && _date_fract == next_date_fract) {
 				if (cp != NULL) {
-					NetworkSend_Command(cp->tile, cp->p1, cp->p2, cp->cmd & ~CMD_FLAGS_MASK, NULL, cp->text, cp->company);
+					NetworkSendCommand(cp->tile, cp->p1, cp->p2, cp->cmd & ~CMD_FLAGS_MASK, NULL, cp->text, cp->company);
 					DEBUG(net, 0, "injecting: %08x; %02x; %02x; %06x; %08x; %08x; %08x; \"%s\" (%s)", _date, _date_fract, (int)_current_company, cp->tile, cp->p1, cp->p2, cp->cmd, cp->text, GetCommandName(cp->cmd));
 					free(cp);
 					cp = NULL;
@@ -1181,9 +942,10 @@ void NetworkGameLoop()
 		if (_frame_counter >= _frame_counter_max) {
 			/* Only check for active clients just before we're going to send out
 			 * the commands so we don't send multiple pause/unpause commands when
-			 * the frame_freq is more than 1 tick. */
+			 * the frame_freq is more than 1 tick. Same with distributing commands. */
 			CheckPauseOnJoin();
 			CheckMinActiveClients();
+			NetworkDistributeCommands();
 		}
 
 		bool send_frame = false;
@@ -1212,12 +974,16 @@ void NetworkGameLoop()
 
 		/* Make sure we are at the frame were the server is (quick-frames) */
 		if (_frame_counter_server > _frame_counter) {
+			/* Run a number of frames; when things go bad, get out. */
 			while (_frame_counter_server > _frame_counter) {
-				if (!NetworkDoClientLoop()) break;
+				if (!ClientNetworkGameSocketHandler::GameLoop()) return;
 			}
 		} else {
 			/* Else, keep on going till _frame_counter_max */
-			if (_frame_counter_max > _frame_counter) NetworkDoClientLoop();
+			if (_frame_counter_max > _frame_counter) {
+				/* Run one frame; if things went bad, get out. */
+				if (!ClientNetworkGameSocketHandler::GameLoop()) return;
+			}
 		}
 	}
 
