@@ -1,4 +1,4 @@
-/* $Id: network_udp.cpp 19094 2010-02-11 14:58:51Z yexo $ */
+/* $Id: network_udp.cpp 22701 2011-07-30 17:45:37Z rubidium $ */
 
 /*
  * This file is part of OpenTTD.
@@ -28,6 +28,9 @@
 #include "../company_base.h"
 #include "../thread/thread.h"
 #include "../rev.h"
+#include "../newgrf_text.h"
+#include "../strings_func.h"
+#include "table/strings.h"
 
 #include "core/udp.h"
 
@@ -36,11 +39,9 @@ static ThreadMutex *_network_udp_mutex = ThreadMutex::New();
 /** Session key to register ourselves to the master server */
 static uint64 _session_key = 0;
 
-enum {
-	ADVERTISE_NORMAL_INTERVAL = 30000, // interval between advertising in ticks (15 minutes)
-	ADVERTISE_RETRY_INTERVAL  =   300, // readvertise when no response after this many ticks (9 seconds)
-	ADVERTISE_RETRY_TIMES     =     3  // give up readvertising after this much failed retries
-};
+static const uint ADVERTISE_NORMAL_INTERVAL = 30000; ///< interval between advertising in ticks (15 minutes)
+static const uint ADVERTISE_RETRY_INTERVAL  =   300; ///< readvertise when no response after this many ticks (9 seconds)
+static const uint ADVERTISE_RETRY_TIMES     =     3; ///< give up readvertising after this much failed retries
 
 NetworkUDPSocketHandler *_udp_client_socket = NULL; ///< udp client socket
 NetworkUDPSocketHandler *_udp_server_socket = NULL; ///< udp server socket
@@ -116,7 +117,7 @@ DEF_UDP_RECEIVE_COMMAND(Server, PACKET_UDP_CLIENT_FIND_SERVER)
 	strecpy(ngi.server_revision, _openttd_revision, lastof(ngi.server_revision));
 
 	Packet packet(PACKET_UDP_SERVER_RESPONSE);
-	this->Send_NetworkGameInfo(&packet, &ngi);
+	this->SendNetworkGameInfo(&packet, &ngi);
 
 	/* Let the client know that we are here */
 	this->SendPacket(&packet, client_addr);
@@ -139,11 +140,40 @@ DEF_UDP_RECEIVE_COMMAND(Server, PACKET_UDP_CLIENT_DETAIL_INFO)
 	NetworkCompanyStats company_stats[MAX_COMPANIES];
 	NetworkPopulateCompanyStats(company_stats);
 
+	/* The minimum company information "blob" size. */
+	static const uint MIN_CI_SIZE = 54;
+	uint max_cname_length = NETWORK_COMPANY_NAME_LENGTH;
+
+	if (Company::GetNumItems() * (MIN_CI_SIZE + NETWORK_COMPANY_NAME_LENGTH) >= (uint)SEND_MTU - packet.size) {
+		/* Assume we can at least put the company information in the packets. */
+		assert(Company::GetNumItems() * MIN_CI_SIZE < (uint)SEND_MTU - packet.size);
+
+		/* At this moment the company names might not fit in the
+		 * packet. Check whether that is really the case. */
+
+		for (;;) {
+			int free = SEND_MTU - packet.size;
+			Company *company;
+			FOR_ALL_COMPANIES(company) {
+				char company_name[NETWORK_COMPANY_NAME_LENGTH];
+				SetDParam(0, company->index);
+				GetString(company_name, STR_COMPANY_NAME, company_name + max_cname_length - 1);
+				free -= MIN_CI_SIZE;
+				free -= (int)strlen(company_name);
+			}
+			if (free >= 0) break;
+
+			/* Try again, with slightly shorter strings. */
+			assert(max_cname_length > 0);
+			max_cname_length--;
+		}
+	}
+
 	Company *company;
 	/* Go through all the companies */
 	FOR_ALL_COMPANIES(company) {
 		/* Send the information */
-		this->Send_CompanyInformation(&packet, company, &company_stats[company->index]);
+		this->SendCompanyInformation(&packet, company, &company_stats[company->index], max_cname_length);
 	}
 
 	this->SendPacket(&packet, client_addr);
@@ -177,20 +207,20 @@ DEF_UDP_RECEIVE_COMMAND(Server, PACKET_UDP_CLIENT_GET_NEWGRFS)
 	if (num_grfs > NETWORK_MAX_GRF_COUNT) return;
 
 	for (i = 0; i < num_grfs; i++) {
-		GRFConfig c;
+		GRFIdentifier c;
 		const GRFConfig *f;
 
-		this->Recv_GRFIdentifier(p, &c);
+		this->ReceiveGRFIdentifier(p, &c);
 
 		/* Find the matching GRF file */
-		f = FindGRFConfig(c.grfid, c.md5sum);
+		f = FindGRFConfig(c.grfid, FGCM_EXACT, c.md5sum);
 		if (f == NULL) continue; // The GRF is unknown to this server
 
 		/* If the reply might exceed the size of the packet, only reply
 		 * the current list and do not send the other data.
 		 * The name could be an empty string, if so take the filename. */
 		packet_len += sizeof(c.grfid) + sizeof(c.md5sum) +
-				min(strlen((!StrEmpty(f->name)) ? f->name : f->filename) + 1, (size_t)NETWORK_GRF_NAME_LENGTH);
+				min(strlen(f->GetName()) + 1, (size_t)NETWORK_GRF_NAME_LENGTH);
 		if (packet_len > SEND_MTU - 4) { // 4 is 3 byte header + grf count in reply
 			break;
 		}
@@ -206,8 +236,8 @@ DEF_UDP_RECEIVE_COMMAND(Server, PACKET_UDP_CLIENT_GET_NEWGRFS)
 		char name[NETWORK_GRF_NAME_LENGTH];
 
 		/* The name could be an empty string, if so take the filename */
-		strecpy(name, (!StrEmpty(in_reply[i]->name)) ? in_reply[i]->name : in_reply[i]->filename, lastof(name));
-		this->Send_GRFIdentifier(&packet, in_reply[i]);
+		strecpy(name, in_reply[i]->GetName(), lastof(name));
+		this->SendGRFIdentifier(&packet, &in_reply[i]->ident);
 		packet.Send_string(name);
 	}
 
@@ -239,7 +269,7 @@ DEF_UDP_RECEIVE_COMMAND(Client, PACKET_UDP_SERVER_RESPONSE)
 	item = NetworkGameListAddItem(*client_addr);
 
 	ClearGRFConfigList(&item->info.grfconfig);
-	this->Recv_NetworkGameInfo(p, &item->info);
+	this->ReceiveNetworkGameInfo(p, &item->info);
 
 	item->info.compatible = true;
 	{
@@ -256,7 +286,7 @@ DEF_UDP_RECEIVE_COMMAND(Client, PACKET_UDP_SERVER_RESPONSE)
 
 		for (c = item->info.grfconfig; c != NULL; c = c->next) {
 			if (c->status == GCS_NOT_FOUND) item->info.compatible = false;
-			if (c->status != GCS_NOT_FOUND || strcmp(c->name, UNKNOWN_GRF_NAME_PLACEHOLDER) != 0) continue;
+			if (c->status != GCS_NOT_FOUND || strcmp(c->GetName(), UNKNOWN_GRF_NAME_PLACEHOLDER) != 0) continue;
 			in_request[in_request_count] = c;
 			in_request_count++;
 		}
@@ -268,7 +298,7 @@ DEF_UDP_RECEIVE_COMMAND(Client, PACKET_UDP_SERVER_RESPONSE)
 
 			packet.Send_uint8(in_request_count);
 			for (i = 0; i < in_request_count; i++) {
-				this->Send_GRFIdentifier(&packet, in_request[i]);
+				this->SendGRFIdentifier(&packet, &in_request[i]->ident);
 			}
 
 			this->SendPacket(&packet, &item->address);
@@ -339,23 +369,22 @@ DEF_UDP_RECEIVE_COMMAND(Client, PACKET_UDP_SERVER_NEWGRFS)
 	if (num_grfs > NETWORK_MAX_GRF_COUNT) return;
 
 	for (i = 0; i < num_grfs; i++) {
-		char *unknown_name;
 		char name[NETWORK_GRF_NAME_LENGTH];
-		GRFConfig c;
+		GRFIdentifier c;
 
-		this->Recv_GRFIdentifier(p, &c);
+		this->ReceiveGRFIdentifier(p, &c);
 		p->Recv_string(name, sizeof(name));
 
 		/* An empty name is not possible under normal circumstances
 		 * and causes problems when showing the NewGRF list. */
 		if (StrEmpty(name)) continue;
 
-		/* Finds the fake GRFConfig for the just read GRF ID and MD5sum tuple.
+		/* Try to find the GRFTextWrapper for the name of this GRF ID and MD5sum tuple.
 		 * If it exists and not resolved yet, then name of the fake GRF is
 		 * overwritten with the name from the reply. */
-		unknown_name = FindUnknownGRFName(c.grfid, c.md5sum, false);
-		if (unknown_name != NULL && strcmp(unknown_name, UNKNOWN_GRF_NAME_PLACEHOLDER) == 0) {
-			ttd_strlcpy(unknown_name, name, NETWORK_GRF_NAME_LENGTH);
+		GRFTextWrapper *unknown_name = FindUnknownGRFName(c.grfid, c.md5sum, false);
+		if (unknown_name != NULL && strcmp(GetGRFStringFromGRFText(unknown_name->text), UNKNOWN_GRF_NAME_PLACEHOLDER) == 0) {
+			AddGRFTextToList(&unknown_name->text, name);
 		}
 	}
 }
@@ -363,17 +392,23 @@ DEF_UDP_RECEIVE_COMMAND(Client, PACKET_UDP_SERVER_NEWGRFS)
 void ClientNetworkUDPSocketHandler::HandleIncomingNetworkGameInfoGRFConfig(GRFConfig *config)
 {
 	/* Find the matching GRF file */
-	const GRFConfig *f = FindGRFConfig(config->grfid, config->md5sum);
+	const GRFConfig *f = FindGRFConfig(config->ident.grfid, FGCM_EXACT, config->ident.md5sum);
 	if (f == NULL) {
 		/* Don't know the GRF, so mark game incompatible and the (possibly)
 		 * already resolved name for this GRF (another server has sent the
 		 * name of the GRF already */
-		config->name   = FindUnknownGRFName(config->grfid, config->md5sum, true);
+		config->name->Release();
+		config->name = FindUnknownGRFName(config->ident.grfid, config->ident.md5sum, true);
+		config->name->AddRef();
 		config->status = GCS_NOT_FOUND;
 	} else {
-		config->filename  = f->filename;
-		config->name      = f->name;
-		config->info      = f->info;
+		config->filename = f->filename;
+		config->name->Release();
+		config->name = f->name;
+		config->name->AddRef();
+		config->info->Release();
+		config->info = f->info;
+		config->info->AddRef();
 	}
 	SetBit(config->flags, GCF_COPY);
 }
@@ -438,9 +473,9 @@ static void NetworkUDPQueryServerThread(void *pntr)
 
 	/* Clear item in gamelist */
 	NetworkGameList *item = CallocT<NetworkGameList>(1);
-	item->address = *info;
 	info->GetAddressAsString(item->info.server_name, lastof(item->info.server_name));
 	strecpy(item->info.hostname, info->GetHostname(), lastof(item->info.hostname));
+	item->address = *info;
 	item->manually = info->manually;
 	NetworkGameListAddItemDelayed(item);
 
@@ -534,8 +569,7 @@ static void NetworkUDPAdvertiseThread(void *pntr)
 void NetworkUDPAdvertise()
 {
 	/* Check if we should send an advertise */
-	if (!_networking || !_network_server || !_network_udp_server || !_settings_client.network.server_advertise)
-		return;
+	if (!_networking || !_network_server || !_network_udp_server || !_settings_client.network.server_advertise) return;
 
 	if (_network_need_advertise) {
 		_network_need_advertise = false;
@@ -543,13 +577,12 @@ void NetworkUDPAdvertise()
 	} else {
 		/* Only send once every ADVERTISE_NORMAL_INTERVAL ticks */
 		if (_network_advertise_retries == 0) {
-			if ((_network_last_advertise_frame + ADVERTISE_NORMAL_INTERVAL) > _frame_counter)
-				return;
+			if ((_network_last_advertise_frame + ADVERTISE_NORMAL_INTERVAL) > _frame_counter) return;
+
 			_network_advertise_retries = ADVERTISE_RETRY_TIMES;
 		}
 
-		if ((_network_last_advertise_frame + ADVERTISE_RETRY_INTERVAL) > _frame_counter)
-			return;
+		if ((_network_last_advertise_frame + ADVERTISE_RETRY_INTERVAL) > _frame_counter) return;
 	}
 
 	_network_advertise_retries--;
