@@ -1,4 +1,4 @@
-/* $Id: newgrf_callbacks.h 17558 2009-09-16 19:10:50Z rubidium $ */
+/* $Id: newgrf_callbacks.h 21238 2010-11-18 14:17:55Z rubidium $ */
 
 /*
  * This file is part of OpenTTD.
@@ -28,8 +28,8 @@ enum CallbackID {
 
 	/* There are no callbacks 0x02 - 0x0F. */
 
-	/** Powered wagons and visual effects. */
-	CBID_TRAIN_WAGON_POWER               = 0x10, // 8 bit callback
+	/** Visual effects and wagon power. */
+	CBID_VEHICLE_VISUAL_EFFECT           = 0x10, // 8 bit callback
 
 	/** Vehicle length, returns the amount of 1/8's the vehicle is shorter for trains and RVs. */
 	CBID_VEHICLE_LENGTH                  = 0x11,
@@ -43,8 +43,10 @@ enum CallbackID {
 	/** Choose a sprite layout to draw, instead of the standard 0-7 range. */
 	CBID_STATION_SPRITE_LAYOUT           = 0x14,
 
-	/** Refit capacity, the passed vehicle needs to have its ->cargo_type set to
-	 * the cargo we are refitting to, returns the new cargo capacity. */
+	/**
+	 * Refit capacity, the passed vehicle needs to have its ->cargo_type set to
+	 * the cargo we are refitting to, returns the new cargo capacity.
+	 */
 	CBID_VEHICLE_REFIT_CAPACITY          = 0x15, // 15 bit callback
 
 	/** Builds articulated engines for trains and RVs. */
@@ -86,8 +88,10 @@ enum CallbackID {
 	/** Called to determine if the given industry type is available */
 	CBID_INDUSTRY_AVAILABLE              = 0x22, // 15 bit callback
 
-	/** This callback is called from vehicle purchase lists. It returns a value to be
-	 * used as a custom string ID in the 0xD000 range. */
+	/**
+	 * This callback is called from vehicle purchase lists. It returns a value to be
+	 * used as a custom string ID in the 0xD000 range.
+	 */
 	CBID_VEHICLE_ADDITIONAL_TEXT         = 0x23,
 
 	/** Called when building a station to customize the tile layout */
@@ -117,8 +121,10 @@ enum CallbackID {
 	/** Called to determine which cargoes an industry should accept. */
 	CBID_INDTILE_ACCEPT_CARGO            = 0x2C, // 15 bit callback
 
-	/** Called to determine if a specific colour map should be used for a vehicle
-	 * instead of the default livery. */
+	/**
+	 * Called to determine if a specific colour map should be used for a vehicle
+	 * instead of the default livery.
+	 */
 	CBID_VEHICLE_COLOUR_MAPPING          = 0x2D, // 15 bit callback
 
 	/** Called to determine how much cargo a town building produces. */
@@ -128,10 +134,12 @@ enum CallbackID {
 	CBID_INDTILE_SHAPE_CHECK             = 0x2F, // 15 bit callback
 
 	/** Called to determine the type (if any) of foundation to draw for industry tile. */
-	CBID_INDUSTRY_DRAW_FOUNDATIONS       = 0x30, // 15 bit callback
+	CBID_INDTILE_DRAW_FOUNDATIONS        = 0x30, // 15 bit callback
 
-	/** Called when the company (or AI) tries to start or stop a vehicle. Mainly
-	 * used for preventing a vehicle from leaving the depot. */
+	/**
+	 * Called when the company (or AI) tries to start or stop a vehicle. Mainly
+	 * used for preventing a vehicle from leaving the depot.
+	 */
 	CBID_VEHICLE_START_STOP_CHECK        = 0x31, // 15 bit callback, but 0xFF test is done with 8 bit
 
 	/** Called for every vehicle every 32 days (not all on same date though). */
@@ -146,8 +154,10 @@ enum CallbackID {
 	/** Called monthly on production changes, so it can be adjusted more frequently */
 	CBID_INDUSTRY_MONTHLYPROD_CHANGE     = 0x35, // 15 bit callback
 
-	/** Called to modify various vehicle properties. Callback parameter 1
-	 * specifies the property index, as used in Action 0, to change. */
+	/**
+	 * Called to modify various vehicle properties. Callback parameter 1
+	 * specifies the property index, as used in Action 0, to change.
+	 */
 	CBID_VEHICLE_MODIFY_PROPERTY         = 0x36, // 8/15 bit depends on queried property
 
 	/** Called to determine text to display after cargo name */
@@ -166,7 +176,7 @@ enum CallbackID {
 	CBID_INDUSTRY_SPECIAL_EFFECT         = 0x3B, // 15 bit callback
 
 	/** Called to determine if industry can alter the ground below industry tile */
-	CBID_INDUSTRY_AUTOSLOPE              = 0x3C, // 15 bit callback
+	CBID_INDTILE_AUTOSLOPE               = 0x3C, // 15 bit callback
 
 	/** Called to determine if the industry can still accept or refuse more cargo arrival */
 	CBID_INDUSTRY_REFUSE_CARGO           = 0x3D, // 15 bit callback
@@ -195,7 +205,7 @@ enum CallbackID {
 	CBID_NEW_SIGNALS_SPRITE_DRAW         = 0x146, // 15 bit callback, not implemented
 
 	/** Add an offset to the default sprite numbers to show another sprite. */
-	CBID_CANALS_SPRITE_OFFSET            = 0x147, // 15 bit callback, not implemented
+	CBID_CANALS_SPRITE_OFFSET            = 0x147, // 15 bit callback
 
 	/** Called when a cargo type specified in property 20 is accepted. */
 	CBID_HOUSE_WATCHED_CARGO_ACCEPTED    = 0x148, // 15 bit callback, not implemented
@@ -220,6 +230,48 @@ enum CallbackID {
 
 	/** Called to determine if one can alter the ground below a house tile */
 	CBID_HOUSE_AUTOSLOPE                 = 0x14F, // 15 bit callback
+
+	/** Called to determine the type (if any) of foundation to draw for an airport tile. */
+	CBID_AIRPTILE_DRAW_FOUNDATIONS       = 0x150, // 15 bit callback
+
+	/** Called for periodically starting or stopping the animation. */
+	CBID_AIRPTILE_ANIM_START_STOP        = 0x152, // 15 bit callback
+
+	/** Called to determine airport tile next animation frame. */
+	CBID_AIRPTILE_ANIM_NEXT_FRAME        = 0x153, // 15 bit callback
+
+	/** Called to indicate how long the current animation frame should last. */
+	CBID_AIRPTILE_ANIMATION_SPEED        = 0x154, // 8 bit callback
+
+	/**
+	 * This callback is called from airport list. It returns a value to be
+	 * used as a custom string ID in the 0xD000 range.
+	 */
+	CBID_AIRPORT_ADDITIONAL_TEXT         = 0x155, // 15 bit callback
+
+	/** Called to determine text to show as airport layout name. */
+	CBID_AIRPORT_LAYOUT_NAME             = 0x156, // 15 bit callback
+
+	/** Callback done for each tile of an object to check the slope. */
+	CBID_OBJECT_LAND_SLOPE_CHECK         = 0x157, // 15 bit callback
+
+	/** Determine the next animation frame for a house. */
+	CBID_OBJECT_ANIMATION_NEXT_FRAME     = 0x158, // 15 bit callback
+
+	/** Called for periodically starting or stopping the animation. */
+	CBID_OBJECT_ANIMATION_START_STOP     = 0x159, // 15 bit callback
+
+	/** Called to indicate how long the current animation frame should last. */
+	CBID_OBJECT_ANIMATION_SPEED          = 0x15A, // 8 bit callback
+
+	/** Called to determine the colour of a town building. */
+	CBID_OBJECT_COLOUR                   = 0x15B, // 15 bit callback
+
+	/** Called to determine more text in the fund object window */
+	CBID_OBJECT_FUND_MORE_TEXT           = 0x15C, // 15 bit callback
+
+	/** Called to determine if one can alter the ground below an object tile */
+	CBID_OBJECT_AUTOSLOPE                = 0x15D, // 15 bit callback
 };
 
 /**
@@ -227,7 +279,7 @@ enum CallbackID {
  * Some callbacks are always used and don't have a mask.
  */
 enum VehicleCallbackMask {
-	CBM_TRAIN_WAGON_POWER      = 0, ///< Powered wagons (trains only)
+	CBM_VEHICLE_VISUAL_EFFECT  = 0, ///< Visual effects and wagon power (trains, road vehicles and ships)
 	CBM_VEHICLE_LENGTH         = 1, ///< Vehicle length (trains and road vehicles)
 	CBM_VEHICLE_LOAD_AMOUNT    = 2, ///< Load amount
 	CBM_VEHICLE_REFIT_CAPACITY = 3, ///< Cargo capacity after refit
@@ -316,11 +368,32 @@ enum IndustryTileCallbackMask {
 };
 
 /**
- * Different values for Callback result evaluations
+ * Callback masks for objects
  */
-enum {
-	CALLBACK_FAILED              = 0xFFFF,  ///< Result of a failed callback.
-	CALLBACK_HOUSEPRODCARGO_END  = 0x20FF,  ///< Sentinel indicating that the loop for CBID_HOUSE_PRODUCE_CARGO has ended
+enum ObjectCallbackMask {
+	CBM_OBJ_SLOPE_CHECK               =  0, ///< decides slope suitability
+	CBM_OBJ_ANIMATION_NEXT_FRAME      =  1, ///< decides next animation frame
+	CBM_OBJ_ANIMATION_SPEED           =  2, ///< decides animation speed
+	CBM_OBJ_COLOUR                    =  3, ///< decide the color of the building
+	CBM_OBJ_FUND_MORE_TEXT            =  4, ///< additional text in fund window
+	CBM_OBJ_AUTOSLOPE                 =  5, ///< decides allowance of autosloping
 };
 
+/**
+ * Callback masks for airport tiles
+ */
+enum AirportTileCallbackMask {
+	CBM_AIRT_ANIM_NEXT_FRAME          = 0,  ///< decides next animation frame
+	CBM_AIRT_ANIM_SPEED               = 1,  ///< decides animation speed
+	CBM_AIRT_SHAPE_CHECK              = 4,  ///< decides slope suitability
+	CBM_AIRT_DRAW_FOUNDATIONS         = 5,  ///< decides if default foundations need to be drawn
+	CBM_AIRT_AUTOSLOPE                = 6,  ///< decides allowance of autosloping
+};
+
+/**
+ * Different values for Callback result evaluations
+ */
+static const uint CALLBACK_FAILED              = 0xFFFF; ///< Result of a failed callback.
+static const uint CALLBACK_HOUSEPRODCARGO_END  = 0x20FF; ///< Sentinel indicating that the loop for CBID_HOUSE_PRODUCE_CARGO has ended
+
 #endif /* NEWGRF_CALLBACKS_H */
