@@ -1,4 +1,4 @@
-/* $Id: newgrf_engine.cpp 20608 2010-08-24 00:03:26Z rubidium $ */
+/* $Id: newgrf_engine.cpp 22653 2011-07-10 20:29:19Z rubidium $ */
 
 /*
  * This file is part of OpenTTD.
@@ -16,16 +16,14 @@
 #include "company_func.h"
 #include "newgrf.h"
 #include "newgrf_cargo.h"
-#include "newgrf_engine.h"
 #include "newgrf_spritegroup.h"
 #include "date_func.h"
 #include "vehicle_func.h"
 #include "core/random_func.hpp"
 #include "aircraft.h"
-#include "core/smallmap_type.hpp"
 #include "station_base.h"
-#include "engine_base.h"
 #include "company_base.h"
+#include "newgrf_railtype.h"
 
 struct WagonOverride {
 	EngineID *train_id;
@@ -91,12 +89,12 @@ void UnloadWagonOverrides(Engine *e)
 void SetCustomEngineSprites(EngineID engine, byte cargo, const SpriteGroup *group)
 {
 	Engine *e = Engine::Get(engine);
-	assert(cargo < lengthof(e->group));
+	assert(cargo < lengthof(e->grf_prop.spritegroup));
 
-	if (e->group[cargo] != NULL) {
+	if (e->grf_prop.spritegroup[cargo] != NULL) {
 		grfmsg(6, "SetCustomEngineSprites: engine %d cargo %d already has group -- replacing", engine, cargo);
 	}
-	e->group[cargo] = group;
+	e->grf_prop.spritegroup[cargo] = group;
 }
 
 
@@ -109,7 +107,7 @@ void SetCustomEngineSprites(EngineID engine, byte cargo, const SpriteGroup *grou
 void SetEngineGRF(EngineID engine, const GRFFile *file)
 {
 	Engine *e = Engine::Get(engine);
-	e->grffile = file;
+	e->grf_prop.grffile = file;
 }
 
 
@@ -120,7 +118,7 @@ void SetEngineGRF(EngineID engine, const GRFFile *file)
  */
 const GRFFile *GetEngineGRF(EngineID engine)
 {
-	return Engine::Get(engine)->grffile;
+	return Engine::Get(engine)->grf_prop.grffile;
 }
 
 
@@ -154,7 +152,7 @@ static int MapOldSubType(const Vehicle *v)
 
 
 /* TTDP style aircraft movement states for GRF Action 2 Var 0xE2 */
-enum {
+enum TTDPAircraftMovementStates {
 	AMS_TTDP_HANGAR,
 	AMS_TTDP_TO_HANGAR,
 	AMS_TTDP_TO_PAD1,
@@ -196,7 +194,7 @@ static byte MapAircraftMovementState(const Aircraft *v)
 	const Station *st = GetTargetAirportIfValid(v);
 	if (st == NULL) return AMS_TTDP_FLIGHT_TO_TOWER;
 
-	const AirportFTAClass *afc = st->Airport();
+	const AirportFTAClass *afc = st->airport.GetFTA();
 	uint16 amdflag = afc->MovingData(v->pos)->flag;
 
 	switch (v->state) {
@@ -241,7 +239,7 @@ static byte MapAircraftMovementState(const Aircraft *v)
 		case HELIPAD1:
 		case HELIPAD2:
 		case HELIPAD3:
-		case HELIPAD4: // Will only occur for helicopters.
+			/* Will only occur for helicopters.*/
 			if (amdflag & AMED_HELI_LOWER) return AMS_TTDP_HELI_LAND_AIRPORT; // Descending.
 			if (amdflag & AMED_SLOWTURN)   return AMS_TTDP_FLIGHT_TO_TOWER;   // Still hasn't started descent.
 			return AMS_TTDP_TO_JUNCTION; // On the ground.
@@ -290,7 +288,7 @@ static byte MapAircraftMovementState(const Aircraft *v)
 
 
 /* TTDP style aircraft movement action for GRF Action 2 Var 0xE6 */
-enum {
+enum TTDPAircraftMovementActions {
 	AMA_TTDP_IN_HANGAR,
 	AMA_TTDP_ON_PAD1,
 	AMA_TTDP_ON_PAD2,
@@ -339,7 +337,6 @@ static byte MapAircraftMovementAction(const Aircraft *v)
 		case TERM7:
 		case TERM8:
 		case HELIPAD3:
-		case HELIPAD4:
 			return (v->current_order.IsType(OT_LOADING)) ? AMA_TTDP_ON_PAD3 : AMA_TTDP_LANDING_TO_PAD3;
 
 		case TAKEOFF:      // Moving to takeoff position
@@ -366,15 +363,6 @@ static byte MapAircraftMovementAction(const Aircraft *v)
 }
 
 
-/* TTDP airport types. Used to map our types to TTDPatch's */
-enum {
-	ATP_TTDP_SMALL,
-	ATP_TTDP_LARGE,
-	ATP_TTDP_HELIPORT,
-	ATP_TTDP_OILRIG,
-};
-
-
 /* Vehicle Resolver Functions */
 static inline const Vehicle *GRV(const ResolverObject *object)
 {
@@ -404,7 +392,8 @@ static inline const Vehicle *GRV(const ResolverObject *object)
 						}
 					}
 					if (v == NULL) v = self;
-				} break;
+					break;
+				}
 			}
 			uint32 count = GB(object->count, 0, 4);
 			if (count == 0) count = GetRegister(0x100);
@@ -450,10 +439,8 @@ static uint8 LiveryHelper(EngineID engine, const Vehicle *v)
 	if (v == NULL) {
 		if (!Company::IsValidID(_current_company)) return 0;
 		l = GetEngineLivery(engine, _current_company, INVALID_ENGINE, NULL, LIT_ALL);
-	} else if (v->type == VEH_TRAIN) {
-		l = GetEngineLivery(v->engine_type, v->owner, Train::From(v)->tcache.first_engine, v, LIT_ALL);
-	} else if (v->type == VEH_ROAD) {
-		l = GetEngineLivery(v->engine_type, v->owner, RoadVehicle::From(v)->rcache.first_engine, v, LIT_ALL);
+	} else if (v->IsGroundVehicle()) {
+		l = GetEngineLivery(v->engine_type, v->owner, v->GetGroundVehicleCache()->first_engine, v, LIT_ALL);
 	} else {
 		l = GetEngineLivery(v->engine_type, v->owner, INVALID_ENGINE, v, LIT_ALL);
 	}
@@ -487,25 +474,6 @@ static uint32 PositionHelper(const Vehicle *v, bool consecutive)
 	return chain_before | chain_after << 8 | (chain_before + chain_after + consecutive) << 16;
 }
 
-byte MapAirportTypeToTTDType(byte ottd_type)
-{
-	switch (ottd_type) {
-		/* Note, Helidepot and Helistation are treated as small airports
-		 * as they are at ground level. */
-		case AT_HELIDEPOT:
-		case AT_HELISTATION:
-		case AT_COMMUTER:
-		case AT_SMALL:         return ATP_TTDP_SMALL;
-		case AT_METROPOLITAN:
-		case AT_INTERNATIONAL:
-		case AT_INTERCON:
-		case AT_LARGE:         return ATP_TTDP_LARGE;
-		case AT_HELIPORT:      return ATP_TTDP_HELIPORT;
-		case AT_OILRIG:        return ATP_TTDP_OILRIG;
-		default:               return ATP_TTDP_LARGE;
-	}
-}
-
 static uint32 VehicleGetVariable(const ResolverObject *object, byte variable, byte parameter, bool *available)
 {
 	Vehicle *v = const_cast<Vehicle*>(GRV(object));
@@ -542,21 +510,21 @@ static uint32 VehicleGetVariable(const ResolverObject *object, byte variable, by
 			return GetEngineGRFID(v->engine_type);
 
 		case 0x40: // Get length of consist
-			if (!HasBit(v->vcache.cache_valid, 0)) {
-				v->vcache.cached_var40 = PositionHelper(v, false);
-				SetBit(v->vcache.cache_valid, 0);
+			if (!HasBit(v->grf_cache.cache_valid, NCVV_POSITION_CONSIST_LENGTH)) {
+				v->grf_cache.position_consist_length = PositionHelper(v, false);
+				SetBit(v->grf_cache.cache_valid, NCVV_POSITION_CONSIST_LENGTH);
 			}
-			return v->vcache.cached_var40;
+			return v->grf_cache.position_consist_length;
 
 		case 0x41: // Get length of same consecutive wagons
-			if (!HasBit(v->vcache.cache_valid, 1)) {
-				v->vcache.cached_var41 = PositionHelper(v, true);
-				SetBit(v->vcache.cache_valid, 1);
+			if (!HasBit(v->grf_cache.cache_valid, NCVV_POSITION_SAME_ID_LENGTH)) {
+				v->grf_cache.position_same_id_length = PositionHelper(v, true);
+				SetBit(v->grf_cache.cache_valid, NCVV_POSITION_SAME_ID_LENGTH);
 			}
-			return v->vcache.cached_var41;
+			return v->grf_cache.position_same_id_length;
 
 		case 0x42: // Consist cargo information
-			if (!HasBit(v->vcache.cache_valid, 2)) {
+			if (!HasBit(v->grf_cache.cache_valid, NCVV_CONSIST_CARGO_INFORMATION)) {
 				const Vehicle *u;
 				byte cargo_classes = 0;
 				uint8 common_cargos[NUM_CARGO];
@@ -606,17 +574,17 @@ static uint32 VehicleGetVariable(const ResolverObject *object, byte variable, by
 				}
 
 				uint8 common_bitnum = (common_cargo_type == CT_INVALID ? 0xFF : CargoSpec::Get(common_cargo_type)->bitnum);
-				v->vcache.cached_var42 = cargo_classes | (common_bitnum << 8) | (common_subtype << 16) | (user_def_data << 24);
-				SetBit(v->vcache.cache_valid, 2);
+				v->grf_cache.consist_cargo_information = cargo_classes | (common_bitnum << 8) | (common_subtype << 16) | (user_def_data << 24);
+				SetBit(v->grf_cache.cache_valid, NCVV_CONSIST_CARGO_INFORMATION);
 			}
-			return v->vcache.cached_var42;
+			return v->grf_cache.consist_cargo_information;
 
 		case 0x43: // Company information
-			if (!HasBit(v->vcache.cache_valid, 3)) {
-				v->vcache.cached_var43 = v->owner | (Company::IsHumanID(v->owner) ? 0 : 0x10000) | (LiveryHelper(v->engine_type, v) << 24);
-				SetBit(v->vcache.cache_valid, 3);
+			if (!HasBit(v->grf_cache.cache_valid, NCVV_COMPANY_INFORMATION)) {
+				v->grf_cache.company_information = v->owner | (Company::IsHumanID(v->owner) ? 0 : 0x10000) | (LiveryHelper(v->engine_type, v) << 24);
+				SetBit(v->grf_cache.cache_valid, NCVV_COMPANY_INFORMATION);
 			}
-			return v->vcache.cached_var43;
+			return v->grf_cache.company_information;
 
 		case 0x44: // Aircraft information
 			if (v->type != VEH_AIRCRAFT) return UINT_MAX;
@@ -628,8 +596,8 @@ static uint32 VehicleGetVariable(const ResolverObject *object, byte variable, by
 
 				const Station *st = GetTargetAirportIfValid(Aircraft::From(v));
 
-				if (st != NULL) {
-					airporttype = MapAirportTypeToTTDType(st->airport_type);
+				if (st != NULL && st->airport.tile != INVALID_TILE) {
+					airporttype = st->airport.GetSpec()->ttd_airport_type;
 				}
 
 				return (altitude << 8) | airporttype;
@@ -641,7 +609,7 @@ static uint32 VehicleGetVariable(const ResolverObject *object, byte variable, by
 			 * B - current wagon to next wagon, 0 if wagon is last
 			 * T - previous wagon to next wagon, 0 in an S-bend
 			 */
-			if (v->type != VEH_TRAIN && v->type != VEH_ROAD) return 0;
+			if (!v->IsGroundVehicle()) return 0;
 
 			const Vehicle *u_p = v->Previous();
 			const Vehicle *u_n = v->Next();
@@ -672,15 +640,21 @@ static uint32 VehicleGetVariable(const ResolverObject *object, byte variable, by
 		case 0x48: return Engine::Get(v->engine_type)->flags; // Vehicle Type Info
 		case 0x49: return v->build_year;
 
+		case 0x4A: {
+			if (v->type != VEH_TRAIN) return 0;
+			RailType rt = GetTileRailType(v->tile);
+			return (HasPowerOnRail(Train::From(v)->railtype, rt) ? 0x100 : 0) | GetReverseRailTypeTranslation(rt, object->grffile);
+		}
+
 		/* Variables which use the parameter */
 		case 0x60: // Count consist's engine ID occurance
 			//EngineID engine = GetNewEngineID(GetEngineGRF(v->engine_type), v->type, parameter);
-			if (v->type != VEH_TRAIN) return Engine::Get(v->engine_type)->internal_id == parameter;
+			if (v->type != VEH_TRAIN) return Engine::Get(v->engine_type)->grf_prop.local_id == parameter;
 
 			{
 				uint count = 0;
 				for (; v != NULL; v = v->Next()) {
-					if (Engine::Get(v->engine_type)->internal_id == parameter) count++;
+					if (Engine::Get(v->engine_type)->grf_prop.local_id == parameter) count++;
 				}
 				return count;
 			}
@@ -716,7 +690,7 @@ static uint32 VehicleGetVariable(const ResolverObject *object, byte variable, by
 		case 0x0A: return v->current_order.MapOldOrder();
 		case 0x0B: return v->current_order.GetDestination();
 		case 0x0C: return v->GetNumOrders();
-		case 0x0D: return v->cur_order_index;
+		case 0x0D: return v->cur_real_order_index;
 		case 0x10:
 		case 0x11: {
 			uint ticks;
@@ -731,14 +705,26 @@ static uint32 VehicleGetVariable(const ResolverObject *object, byte variable, by
 			}
 			return (variable - 0x80) == 0x10 ? ticks : GB(ticks, 8, 8);
 		}
-		case 0x12: return max(v->date_of_last_service - DAYS_TILL_ORIGINAL_BASE_YEAR, 0);
-		case 0x13: return GB(max(v->date_of_last_service - DAYS_TILL_ORIGINAL_BASE_YEAR, 0), 8, 8);
+		case 0x12: return Clamp(v->date_of_last_service - DAYS_TILL_ORIGINAL_BASE_YEAR, 0, 0xFFFF);
+		case 0x13: return GB(Clamp(v->date_of_last_service - DAYS_TILL_ORIGINAL_BASE_YEAR, 0, 0xFFFF), 8, 8);
 		case 0x14: return v->service_interval;
 		case 0x15: return GB(v->service_interval, 8, 8);
 		case 0x16: return v->last_station_visited;
 		case 0x17: return v->tick_counter;
-		case 0x18: return v->max_speed;
-		case 0x19: return GB(v->max_speed, 8, 8);
+		case 0x18:
+		case 0x19: {
+			uint max_speed;
+			switch (v->type) {
+				case VEH_AIRCRAFT:
+					max_speed = Aircraft::From(v)->GetSpeedOldUnits(); // Convert to old units.
+					break;
+
+				default:
+					max_speed = v->vcache.cached_max_speed;
+					break;
+			}
+			return (variable - 0x80) == 0x18 ? max_speed : GB(max_speed, 8, 8);
+		}
 		case 0x1A: return v->x_pos;
 		case 0x1B: return GB(v->x_pos, 8, 8);
 		case 0x1C: return v->y_pos;
@@ -749,25 +735,25 @@ static uint32 VehicleGetVariable(const ResolverObject *object, byte variable, by
 		case 0x29: return GB(v->cur_image, 8, 8);
 		case 0x32: return v->vehstatus;
 		case 0x33: return 0; // non-existent high byte of vehstatus
-		case 0x34: return v->cur_speed;
-		case 0x35: return GB(v->cur_speed, 8, 8);
+		case 0x34: return v->type == VEH_AIRCRAFT ? (v->cur_speed * 10) / 128 : v->cur_speed;
+		case 0x35: return GB(v->type == VEH_AIRCRAFT ? (v->cur_speed * 10) / 128 : v->cur_speed, 8, 8);
 		case 0x36: return v->subspeed;
 		case 0x37: return v->acceleration;
 		case 0x39: return v->cargo_type;
 		case 0x3A: return v->cargo_cap;
 		case 0x3B: return GB(v->cargo_cap, 8, 8);
-		case 0x3C: return v->cargo.Count();
-		case 0x3D: return GB(v->cargo.Count(), 8, 8);
+		case 0x3C: return ClampToU16(v->cargo.Count());
+		case 0x3D: return GB(ClampToU16(v->cargo.Count()), 8, 8);
 		case 0x3E: return v->cargo.Source();
-		case 0x3F: return v->cargo.DaysInTransit();
-		case 0x40: return v->age;
-		case 0x41: return GB(v->age, 8, 8);
-		case 0x42: return v->max_age;
-		case 0x43: return GB(v->max_age, 8, 8);
+		case 0x3F: return ClampU(v->cargo.DaysInTransit(), 0, 0xFF);
+		case 0x40: return ClampToU16(v->age);
+		case 0x41: return GB(ClampToU16(v->age), 8, 8);
+		case 0x42: return ClampToU16(v->max_age);
+		case 0x43: return GB(ClampToU16(v->max_age), 8, 8);
 		case 0x44: return Clamp(v->build_year, ORIGINAL_BASE_YEAR, ORIGINAL_MAX_YEAR) - ORIGINAL_BASE_YEAR;
 		case 0x45: return v->unitnumber;
-		case 0x46: return Engine::Get(v->engine_type)->internal_id;
-		case 0x47: return GB(Engine::Get(v->engine_type)->internal_id, 8, 8);
+		case 0x46: return Engine::Get(v->engine_type)->grf_prop.local_id;
+		case 0x47: return GB(Engine::Get(v->engine_type)->grf_prop.local_id, 8, 8);
 		case 0x48:
 			if (v->type != VEH_TRAIN || v->spritenum != 0xFD) return v->spritenum;
 			return HasBit(Train::From(v)->flags, VRF_REVERSE_DIRECTION) ? 0xFE : 0xFD;
@@ -806,16 +792,17 @@ static uint32 VehicleGetVariable(const ResolverObject *object, byte variable, by
 			switch (variable - 0x80) {
 				case 0x62: return t->track;
 				case 0x66: return t->railtype;
-				case 0x73: return t->tcache.cached_veh_length;
-				case 0x74: return t->tcache.cached_power;
-				case 0x75: return GB(t->tcache.cached_power,  8, 24);
-				case 0x76: return GB(t->tcache.cached_power, 16, 16);
-				case 0x77: return GB(t->tcache.cached_power, 24,  8);
+				case 0x73: return t->gcache.cached_veh_length;
+				case 0x74: return t->gcache.cached_power;
+				case 0x75: return GB(t->gcache.cached_power,  8, 24);
+				case 0x76: return GB(t->gcache.cached_power, 16, 16);
+				case 0x77: return GB(t->gcache.cached_power, 24,  8);
 				case 0x7C: return t->First()->index;
 				case 0x7D: return GB(t->First()->index, 8, 8);
 				case 0x7F: return 0; // Used for vehicle reversing hack in TTDP
 			}
-		} break;
+			break;
+		}
 
 		case VEH_ROAD: {
 			RoadVehicle *rv = RoadVehicle::From(v);
@@ -828,7 +815,8 @@ static uint32 VehicleGetVariable(const ResolverObject *object, byte variable, by
 				case 0x68: return rv->crashed_ctr;
 				case 0x69: return GB(rv->crashed_ctr, 8, 8);
 			}
-		} break;
+			break;
+		}
 
 		case VEH_AIRCRAFT: {
 			Aircraft *a = Aircraft::From(v);
@@ -837,12 +825,13 @@ static uint32 VehicleGetVariable(const ResolverObject *object, byte variable, by
 				case 0x63: return a->targetairport;             // Airport to which the action refers
 				case 0x66: return MapAircraftMovementAction(a); // Current movement action
 			}
-		} break;
+			break;
+		}
 
 		default: break;
 	}
 
-	DEBUG(grf, 1, "Unhandled vehicle property 0x%X, type 0x%X", variable, (uint)v->type);
+	DEBUG(grf, 1, "Unhandled vehicle variable 0x%X, type 0x%X", variable, (uint)v->type);
 
 	*available = false;
 	return UINT_MAX;
@@ -889,17 +878,15 @@ static inline void NewVehicleResolver(ResolverObject *res, EngineID engine_type,
 	res->callback        = CBID_NO_CALLBACK;
 	res->callback_param1 = 0;
 	res->callback_param2 = 0;
-	res->last_value      = 0;
-	res->trigger         = 0;
-	res->reseed          = 0;
-	res->count           = 0;
+	res->ResetState();
 
 	const Engine *e = Engine::Get(engine_type);
-	res->grffile         = (e != NULL ? e->grffile : NULL);
+	res->grffile         = (e != NULL ? e->grf_prop.grffile : NULL);
 }
 
 
-/** Retrieve the SpriteGroup for the specified vehicle.
+/**
+ * Retrieve the SpriteGroup for the specified vehicle.
  * If the vehicle is not specified, the purchase list group for the engine is
  * chosen. For trains, an additional engine override lookup is performed.
  * @param engine    Engine type of the vehicle.
@@ -917,26 +904,27 @@ static const SpriteGroup *GetVehicleSpriteGroup(EngineID engine, const Vehicle *
 	} else {
 		cargo = v->cargo_type;
 
-		if (v->type == VEH_TRAIN) {
-			/* We always use cached value, except for callbacks because the override spriteset
+		if (v->IsGroundVehicle()) {
+			/* For trains we always use cached value, except for callbacks because the override spriteset
 			 * to use may be different than the one cached. It happens for callback 0x15 (refit engine),
 			 * as v->cargo_type is temporary changed to the new type */
-			group = use_cache ? Train::From(v)->tcache.cached_override : GetWagonOverrideSpriteSet(v->engine_type, v->cargo_type, Train::From(v)->tcache.first_engine);
-			if (group != NULL) return group;
-		} else if (v->type == VEH_ROAD) {
-			group = GetWagonOverrideSpriteSet(v->engine_type, v->cargo_type, RoadVehicle::From(v)->rcache.first_engine);
+			if (use_cache && v->type == VEH_TRAIN) {
+				group = Train::From(v)->tcache.cached_override;
+			} else {
+				group = GetWagonOverrideSpriteSet(v->engine_type, v->cargo_type, v->GetGroundVehicleCache()->first_engine);
+			}
 			if (group != NULL) return group;
 		}
 	}
 
 	const Engine *e = Engine::Get(engine);
 
-	assert(cargo < lengthof(e->group));
-	group = e->group[cargo];
+	assert(cargo < lengthof(e->grf_prop.spritegroup));
+	group = e->grf_prop.spritegroup[cargo];
 	if (group != NULL) return group;
 
 	/* Fall back to the default set if the selected cargo type is not defined */
-	return e->group[CT_DEFAULT];
+	return e->grf_prop.spritegroup[CT_DEFAULT];
 }
 
 
@@ -1082,8 +1070,9 @@ static void DoTriggerVehicle(Vehicle *v, VehicleTrigger trigger, byte base_rando
 	if (group == NULL) return;
 
 	new_random_bits = Random();
-	v->random_bits &= ~object.reseed;
-	v->random_bits |= (first ? new_random_bits : base_random_bits) & object.reseed;
+	uint32 reseed = object.GetReseedSum(); // The scope only affects triggers, not the reseeding
+	v->random_bits &= ~reseed;
+	v->random_bits |= (first ? new_random_bits : base_random_bits) & reseed;
 
 	switch (trigger) {
 		case VEHICLE_TRIGGER_NEW_CARGO:
@@ -1150,10 +1139,10 @@ void TriggerVehicle(Vehicle *v, VehicleTrigger trigger)
 uint ListPositionOfEngine(EngineID engine)
 {
 	const Engine *e = Engine::Get(engine);
-	if (e->grffile == NULL) return e->list_position;
+	if (e->grf_prop.grffile == NULL) return e->list_position;
 
 	/* Crude sorting to group by GRF ID */
-	return (e->grffile->grfid * 256) + e->list_position;
+	return (e->grf_prop.grffile->grfid * 256) + e->list_position;
 }
 
 struct ListOrderChange {
@@ -1190,8 +1179,8 @@ void CommitVehicleListOrderChanges()
 		/* Populate map with current list positions */
 		Engine *e;
 		FOR_ALL_ENGINES_OF_TYPE(e, source_e->type) {
-			if (!_settings_game.vehicle.dynamic_engines || e->grffile == source_e->grffile) {
-				if (e->internal_id == target) target_e = e;
+			if (!_settings_game.vehicle.dynamic_engines || e->grf_prop.grffile == source_e->grf_prop.grffile) {
+				if (e->grf_prop.local_id == target) target_e = e;
 				lptr_map[e->list_position] = e;
 			}
 		}
@@ -1219,3 +1208,45 @@ void CommitVehicleListOrderChanges()
 	/* Clear out the queue */
 	_list_order_changes.Reset();
 }
+
+/**
+ * Resolve an engine's spec and such so we can get a variable.
+ * @param ro    The resolver object to fill.
+ * @param index The vehicle to get the data from.
+ */
+void GetVehicleResolver(ResolverObject *ro, uint index)
+{
+	Vehicle *v = Vehicle::Get(index);
+	NewVehicleResolver(ro, v->engine_type, v);
+}
+
+/**
+ * Fill the grf_cache of the given vehicle.
+ * @param v The vehicle to fill the cache for.
+ */
+void FillNewGRFVehicleCache(const Vehicle *v)
+{
+	ResolverObject ro;
+	memset(&ro, 0, sizeof(ro));
+	GetVehicleResolver(&ro, v->index);
+
+	/* These variables we have to check; these are the ones with a cache. */
+	static const int cache_entries[][2] = {
+		{ 0x40, NCVV_POSITION_CONSIST_LENGTH },
+		{ 0x41, NCVV_POSITION_SAME_ID_LENGTH },
+		{ 0x42, NCVV_CONSIST_CARGO_INFORMATION },
+		{ 0x43, NCVV_COMPANY_INFORMATION },
+	};
+	assert_compile(NCVV_END == lengthof(cache_entries));
+
+	/* Resolve all the variables, so their caches are set. */
+	for (size_t i = 0; i < lengthof(cache_entries); i++) {
+		/* Only resolve when the cache isn't valid. */
+		if (HasBit(v->grf_cache.cache_valid, cache_entries[i][1])) continue;
+		bool stub;
+		ro.GetVariable(&ro, cache_entries[i][0], 0, &stub);
+	}
+
+	/* Make sure really all bits are set. */
+	assert(v->grf_cache.cache_valid == (1 << NCVV_END) - 1);
+}
