@@ -1,4 +1,4 @@
-/* $Id: ship_cmd.cpp 19665 2010-04-17 22:27:49Z rubidium $ */
+/* $Id: ship_cmd.cpp 22391 2011-04-30 20:42:50Z rubidium $ */
 
 /*
  * This file is part of OpenTTD.
@@ -13,7 +13,6 @@
 #include "ship.h"
 #include "landscape.h"
 #include "timetable.h"
-#include "command_func.h"
 #include "news_func.h"
 #include "company_func.h"
 #include "pathfinder/npf/npf_func.h"
@@ -25,23 +24,16 @@
 #include "newgrf_sound.h"
 #include "spritecache.h"
 #include "strings_func.h"
-#include "functions.h"
 #include "window_func.h"
 #include "date_func.h"
 #include "vehicle_func.h"
 #include "sound_func.h"
-#include "autoreplace_gui.h"
-#include "effectvehicle_func.h"
-#include "effectvehicle_base.h"
 #include "ai/ai.hpp"
 #include "pathfinder/opf/opf_ship.h"
-#include "landscape_type.h"
 #include "engine_base.h"
-#include "engine_func.h"
 #include "company_base.h"
 
 #include "table/strings.h"
-#include "table/sprites.h"
 
 static const uint16 _ship_sprites[] = {0x0E5D, 0x0E55, 0x0E65, 0x0E6D};
 
@@ -73,7 +65,8 @@ void DrawShipEngine(int left, int right, int preferred_x, int y, EngineID engine
 	DrawSprite(sprite, pal, preferred_x, y);
 }
 
-/** Get the size of the sprite of a ship sprite heading west (used for lists)
+/**
+ * Get the size of the sprite of a ship sprite heading west (used for lists)
  * @param engine The engine to get the sprite from
  * @param width The width of the sprite
  * @param height The height of the sprite
@@ -157,17 +150,25 @@ static void CheckIfShipNeedsService(Vehicle *v)
 	SetWindowWidgetDirty(WC_VEHICLE_VIEW, v->index, VVW_WIDGET_START_STOP_VEH);
 }
 
+void Ship::UpdateCache()
+{
+	this->vcache.cached_max_speed = GetVehicleProperty(this, PROP_SHIP_SPEED, ShipVehInfo(this->engine_type)->max_speed);
+
+	this->UpdateVisualEffect();
+}
+
 Money Ship::GetRunningCost() const
 {
 	const Engine *e = Engine::Get(this->engine_type);
 	uint cost_factor = GetVehicleProperty(this, PROP_SHIP_RUNNING_COST_FACTOR, e->u.ship.running_cost);
-	return GetPrice(PR_RUNNING_SHIP, cost_factor, e->grffile);
+	return GetPrice(PR_RUNNING_SHIP, cost_factor, e->grf_prop.grffile);
 }
 
 void Ship::OnNewDay()
 {
-	if ((++this->day_counter & 7) == 0)
+	if ((++this->day_counter & 7) == 0) {
 		DecreaseVehicleValue(this);
+	}
 
 	CheckVehicleBreakdown(this);
 	AgeVehicle(this);
@@ -206,42 +207,10 @@ Trackdir Ship::GetVehicleTrackdir() const
 	return TrackDirectionToTrackdir(FindFirstTrack(this->state), this->direction);
 }
 
-static void HandleBrokenShip(Vehicle *v)
-{
-	if (v->breakdown_ctr != 1) {
-		v->breakdown_ctr = 1;
-		v->cur_speed = 0;
-
-		if (v->breakdowns_since_last_service != 255)
-			v->breakdowns_since_last_service++;
-
-		v->MarkDirty();
-		SetWindowDirty(WC_VEHICLE_VIEW, v->index);
-		SetWindowDirty(WC_VEHICLE_DETAILS, v->index);
-
-		if (!PlayVehicleSound(v, VSE_BREAKDOWN)) {
-			SndPlayVehicleFx((_settings_game.game_creation.landscape != LT_TOYLAND) ?
-				SND_10_TRAIN_BREAKDOWN : SND_3A_COMEDY_BREAKDOWN_2, v);
-		}
-
-		if (!(v->vehstatus & VS_HIDDEN)) {
-			EffectVehicle *u = CreateEffectVehicleRel(v, 4, 4, 5, EV_BREAKDOWN_SMOKE);
-			if (u != NULL) u->animation_state = v->breakdown_delay * 2;
-		}
-	}
-
-	if (!(v->tick_counter & 1)) {
-		if (!--v->breakdown_delay) {
-			v->breakdown_ctr = 0;
-			v->MarkDirty();
-			SetWindowDirty(WC_VEHICLE_VIEW, v->index);
-		}
-	}
-}
-
 void Ship::MarkDirty()
 {
 	this->UpdateViewport(false, false);
+	this->UpdateCache();
 }
 
 static void PlayShipSound(const Vehicle *v)
@@ -264,7 +233,7 @@ TileIndex Ship::GetOrderStationLocation(StationID station)
 	if (st->dock_tile != INVALID_TILE) {
 		return TILE_ADD(st->dock_tile, ToTileIndexDiff(GetDockOffset(st->dock_tile)));
 	} else {
-		this->IncrementOrderIndex();
+		this->IncrementRealOrderIndex();
 		return 0;
 	}
 }
@@ -292,20 +261,21 @@ void Ship::UpdateDeltaXY(Direction direction)
 	this->z_extent      = 6;
 }
 
-void RecalcShipStuff(Vehicle *v)
-{
-	v->UpdateViewport(false, true);
-	SetWindowDirty(WC_VEHICLE_DEPOT, v->tile);
-}
-
 static const TileIndexDiffC _ship_leave_depot_offs[] = {
 	{-1,  0},
 	{ 0, -1}
 };
 
-static void CheckShipLeaveDepot(Ship *v)
+static bool CheckShipLeaveDepot(Ship *v)
 {
-	if (!v->IsInDepot()) return;
+	if (!v->IsInDepot()) return false;
+
+	/* We are leaving a depot, but have to go to the exact same one; re-enter */
+	if (v->current_order.IsType(OT_GOTO_DEPOT) &&
+			IsShipDepotTile(v->tile) && GetDepotIndex(v->tile) == v->current_order.GetDestination()) {
+		VehicleEnterDepot(v);
+		return true;
+	}
 
 	TileIndex tile = v->tile;
 	Axis axis = GetShipDepotAxis(tile);
@@ -317,19 +287,22 @@ static void CheckShipLeaveDepot(Ship *v)
 	} else if (DiagdirReachesTracks((DiagDirection)(axis + 2)) & GetTileShipTrackStatus(TILE_ADD(tile, -2 * ToTileIndexDiff(_ship_leave_depot_offs[axis])))) {
 		v->direction = AxisToDirection(axis);
 	} else {
-		return;
+		return false;
 	}
 
 	v->state = AxisToTrackBits(axis);
 	v->vehstatus &= ~VS_HIDDEN;
 
 	v->cur_speed = 0;
-	RecalcShipStuff(v);
+	v->UpdateViewport(false, true);
+	SetWindowDirty(WC_VEHICLE_DEPOT, v->tile);
 
 	PlayShipSound(v);
 	VehicleServiceInDepot(v);
 	InvalidateWindowData(WC_VEHICLE_DEPOT, v->tile);
 	SetWindowClassesDirty(WC_SHIPS_LIST);
+
+	return false;
 }
 
 static bool ShipAccelerate(Vehicle *v)
@@ -337,17 +310,16 @@ static bool ShipAccelerate(Vehicle *v)
 	uint spd;
 	byte t;
 
-	spd = min(v->cur_speed + 1, GetVehicleProperty(v, PROP_SHIP_SPEED, v->max_speed));
+	spd = min(v->cur_speed + 1, v->vcache.cached_max_speed);
 
 	/* updates statusbar only if speed have changed to save CPU time */
 	if (spd != v->cur_speed) {
 		v->cur_speed = spd;
-		if (_settings_client.gui.vehicle_speed)
-			SetWindowWidgetDirty(WC_VEHICLE_VIEW, v->index, VVW_WIDGET_START_STOP_VEH);
+		SetWindowWidgetDirty(WC_VEHICLE_VIEW, v->index, VVW_WIDGET_START_STOP_VEH);
 	}
 
-	/* Decrease somewhat when turning */
-	if (!(v->direction & 1)) spd = spd * 3 / 4;
+	/* Convert direction-indepenent speed into direction-dependent speed. (old movement method) */
+	spd = v->GetOldAdvanceSpeed(spd);
 
 	if (spd == 0) return false;
 	if ((byte)++spd == 0) return true;
@@ -357,6 +329,11 @@ static bool ShipAccelerate(Vehicle *v)
 	return (t < v->progress);
 }
 
+/**
+ * Ship arrives at a dock. If it is the first time, send out a news item.
+ * @param v  Ship that arrived.
+ * @param st Station being visited.
+ */
 static void ShipArrivesAt(const Vehicle *v, Station *st)
 {
 	/* Check if station was ever visited before */
@@ -375,19 +352,26 @@ static void ShipArrivesAt(const Vehicle *v, Station *st)
 }
 
 
-/** returns the track to choose on the next tile, or -1 when it's better to
+/**
+ * returns the track to choose on the next tile, or -1 when it's better to
  * reverse. The tile given is the tile we are about to enter, enterdir is the
- * direction in which we are entering the tile */
-static Track ChooseShipTrack(const Ship *v, TileIndex tile, DiagDirection enterdir, TrackBits tracks)
+ * direction in which we are entering the tile
+ */
+static Track ChooseShipTrack(Ship *v, TileIndex tile, DiagDirection enterdir, TrackBits tracks)
 {
 	assert(IsValidDiagDirection(enterdir));
 
+	bool path_found = true;
+	Track track;
 	switch (_settings_game.pf.pathfinder_for_ships) {
-		case VPF_OPF: return OPFShipChooseTrack(v, tile, enterdir, tracks);
-		case VPF_NPF: return NPFShipChooseTrack(v, tile, enterdir, tracks);
-		case VPF_YAPF: return YapfShipChooseTrack(v, tile, enterdir, tracks);
+		case VPF_OPF: track = OPFShipChooseTrack(v, tile, enterdir, tracks, path_found); break;
+		case VPF_NPF: track = NPFShipChooseTrack(v, tile, enterdir, tracks, path_found); break;
+		case VPF_YAPF: track = YapfShipChooseTrack(v, tile, enterdir, tracks, path_found); break;
 		default: NOT_REACHED();
 	}
+
+	v->HandlePathfindingResult(path_found);
+	return track;
 }
 
 static const Direction _new_vehicle_direction_table[] = {
@@ -462,13 +446,7 @@ static void ShipController(Ship *v)
 	v->tick_counter++;
 	v->current_order_time++;
 
-	if (v->breakdown_ctr != 0) {
-		if (v->breakdown_ctr <= 2) {
-			HandleBrokenShip(v);
-			return;
-		}
-		if (!v->current_order.IsType(OT_LOADING)) v->breakdown_ctr--;
-	}
+	if (v->HandleBreakdown()) return;
 
 	if (v->vehstatus & VS_STOPPED) return;
 
@@ -477,7 +455,9 @@ static void ShipController(Ship *v)
 
 	if (v->current_order.IsType(OT_LOADING)) return;
 
-	CheckShipLeaveDepot(v);
+	if (CheckShipLeaveDepot(v)) return;
+
+	v->ShowVisualEffect();
 
 	if (!ShipAccelerate(v)) return;
 
@@ -506,7 +486,7 @@ static void ShipController(Ship *v)
 						/* We got within 3 tiles of our target buoy, so let's skip to our
 						 * next order */
 						UpdateVehicleTimetable(v, true);
-						v->IncrementOrderIndex();
+						v->IncrementRealOrderIndex();
 						v->current_order.MakeDummy();
 					} else {
 						/* Non-buoy orders really need to reach the tile */
@@ -526,7 +506,7 @@ static void ShipController(Ship *v)
 									v->BeginLoading();
 								} else { // leave stations without docks right aways
 									v->current_order.MakeLeaveStation();
-									v->IncrementOrderIndex();
+									v->IncrementRealOrderIndex();
 								}
 							}
 						}
@@ -601,39 +581,18 @@ bool Ship::Tick()
 	return true;
 }
 
-/** Build a ship.
- * @param tile tile of depot where ship is built
- * @param flags type of operation
- * @param p1 ship type being built (engine)
- * @param p2 unused
- * @param text unused
- * @return the cost of this operation or an error
+/**
+ * Build a ship.
+ * @param tile     tile of the depot where ship is built.
+ * @param flags    type of operation.
+ * @param e        the engine to build.
+ * @param data     unused.
+ * @param ret[out] the vehicle that has been built.
+ * @return the cost of this operation or an error.
  */
-CommandCost CmdBuildShip(TileIndex tile, DoCommandFlag flags, uint32 p1, uint32 p2, const char *text)
+CommandCost CmdBuildShip(TileIndex tile, DoCommandFlag flags, const Engine *e, uint16 data, Vehicle **ret)
 {
-	EngineID eid = GB(p1, 0, 16);
-	UnitID unit_num;
-
-	if (!IsEngineBuildable(eid, VEH_SHIP, _current_company)) return_cmd_error(STR_ERROR_SHIP_NOT_AVAILABLE);
-
-	const Engine *e = Engine::Get(eid);
-	CommandCost value(EXPENSES_NEW_VEHICLES, e->GetCost());
-
-	/* Engines without valid cargo should not be available */
-	if (e->GetDefaultCargoType() == CT_INVALID) return CMD_ERROR;
-
-	if (flags & DC_QUERY_COST) return value;
-
-	/* The ai_new queries the vehicle cost before building the route,
-	 * so we must check against cheaters no sooner than now. --pasky */
-	if (!IsShipDepotTile(tile)) return CMD_ERROR;
-	if (!IsTileOwner(tile, _current_company)) return CMD_ERROR;
-
-	unit_num = (flags & DC_AUTOREPLACE) ? 0 : GetFreeUnitNumber(VEH_SHIP);
-
-	if (!Vehicle::CanAllocateItem() || unit_num > _settings_game.vehicle.max_ships)
-		return_cmd_error(STR_ERROR_TOO_MANY_VEHICLES_IN_GAME);
-
+	tile = GetShipDepotNorthTile(tile);
 	if (flags & DC_EXEC) {
 		int x;
 		int y;
@@ -641,7 +600,7 @@ CommandCost CmdBuildShip(TileIndex tile, DoCommandFlag flags, uint32 p1, uint32
 		const ShipVehicleInfo *svi = &e->u.ship;
 
 		Ship *v = new Ship();
-		v->unitnumber = unit_num;
+		*ret = v;
 
 		v->owner = _current_company;
 		v->tile = tile;
@@ -657,11 +616,9 @@ CommandCost CmdBuildShip(TileIndex tile, DoCommandFlag flags, uint32 p1, uint32
 		v->spritenum = svi->image_index;
 		v->cargo_type = e->GetDefaultCargoType();
 		v->cargo_cap = svi->capacity;
-		v->value = value.GetCost();
 
 		v->last_station_visited = INVALID_STATION;
-		v->max_speed = svi->max_speed;
-		v->engine_type = eid;
+		v->engine_type = e->index;
 
 		v->reliability = e->reliability;
 		v->reliability_spd_dec = e->reliability_spd_dec;
@@ -676,6 +633,8 @@ CommandCost CmdBuildShip(TileIndex tile, DoCommandFlag flags, uint32 p1, uint32
 		v->cur_image = SPR_IMG_QUERY;
 		v->random_bits = VehicleRandomBits();
 
+		v->UpdateCache();
+
 		if (e->flags & ENGINE_EXCLUSIVE_PREVIEW) SetBit(v->vehicle_flags, VF_BUILT_AS_PROTOTYPE);
 
 		v->InvalidateNewGRFCacheOfChain();
@@ -685,46 +644,9 @@ CommandCost CmdBuildShip(TileIndex tile, DoCommandFlag flags, uint32 p1, uint32
 		v->InvalidateNewGRFCacheOfChain();
 
 		VehicleMove(v, false);
-
-		InvalidateWindowData(WC_VEHICLE_DEPOT, v->tile);
-		InvalidateWindowClassesData(WC_SHIPS_LIST, 0);
-		SetWindowDirty(WC_COMPANY, v->owner);
-		if (IsLocalCompany()) {
-			InvalidateAutoreplaceWindow(v->engine_type, v->group_id); // updates the replace Ship window
-		}
-
-		Company::Get(_current_company)->num_engines[eid]++;
 	}
 
-	return value;
-}
-
-/** Sell a ship.
- * @param tile unused
- * @param flags type of operation
- * @param p1 vehicle ID to be sold
- * @param p2 unused
- * @param text unused
- * @return the cost of this operation or an error
- */
-CommandCost CmdSellShip(TileIndex tile, DoCommandFlag flags, uint32 p1, uint32 p2, const char *text)
-{
-	Ship *v = Ship::GetIfValid(p1);
-	if (v == NULL || !CheckOwnership(v->owner)) return CMD_ERROR;
-
-	if (v->vehstatus & VS_CRASHED) return_cmd_error(STR_ERROR_CAN_T_SELL_DESTROYED_VEHICLE);
-
-	if (!v->IsStoppedInDepot()) {
-		return_cmd_error(STR_ERROR_SHIP_MUST_BE_STOPPED_IN_DEPOT);
-	}
-
-	CommandCost ret(EXPENSES_NEW_VEHICLES, -v->value);
-
-	if (flags & DC_EXEC) {
-		delete v;
-	}
-
-	return ret;
+	return CommandCost();
 }
 
 bool Ship::FindClosestDepot(TileIndex *location, DestinationID *destination, bool *reverse)
@@ -738,67 +660,3 @@ bool Ship::FindClosestDepot(TileIndex *location, DestinationID *destination, boo
 
 	return true;
 }
-
-/** Send a ship to the depot.
- * @param tile unused
- * @param flags type of operation
- * @param p1 vehicle ID to send to the depot
- * @param p2 various bitmasked elements
- * - p2 bit 0-3 - DEPOT_ flags (see vehicle.h)
- * - p2 bit 8-10 - VLW flag (for mass goto depot)
- * @param text unused
- * @return the cost of this operation or an error
- */
-CommandCost CmdSendShipToDepot(TileIndex tile, DoCommandFlag flags, uint32 p1, uint32 p2, const char *text)
-{
-	if (p2 & DEPOT_MASS_SEND) {
-		/* Mass goto depot requested */
-		if (!ValidVLWFlags(p2 & VLW_MASK)) return CMD_ERROR;
-		return SendAllVehiclesToDepot(VEH_SHIP, flags, p2 & DEPOT_SERVICE, _current_company, (p2 & VLW_MASK), p1);
-	}
-
-	Ship *v = Ship::GetIfValid(p1);
-	if (v == NULL) return CMD_ERROR;
-
-	return v->SendToDepot(flags, (DepotCommand)(p2 & DEPOT_COMMAND_MASK));
-}
-
-
-/** Refits a ship to the specified cargo type.
- * @param tile unused
- * @param flags type of operation
- * @param p1 vehicle ID of the ship to refit
- * @param p2 various bitstuffed elements
- * - p2 = (bit 0-7) - the new cargo type to refit to (p2 & 0xFF)
- * - p2 = (bit 8-15) - the new cargo subtype to refit to
- * - p2 = (bit 16) - refit only this vehicle (ignored)
- * @param text unused
- * @return the cost of this operation or an error
- */
-CommandCost CmdRefitShip(TileIndex tile, DoCommandFlag flags, uint32 p1, uint32 p2, const char *text)
-{
-	CargoID new_cid = GB(p2, 0, 8); // gets the cargo number
-	byte new_subtype = GB(p2, 8, 8);
-
-	Ship *v = Ship::GetIfValid(p1);
-
-	if (v == NULL || !CheckOwnership(v->owner)) return CMD_ERROR;
-	if (!v->IsStoppedInDepot()) return_cmd_error(STR_ERROR_SHIP_MUST_BE_STOPPED_IN_DEPOT);
-	if (v->vehstatus & VS_CRASHED) return_cmd_error(STR_ERROR_CAN_T_REFIT_DESTROYED_VEHICLE);
-
-	/* Check cargo */
-	if (new_cid >= NUM_CARGO) return CMD_ERROR;
-
-	CommandCost cost = RefitVehicle(v, true, new_cid, new_subtype, flags);
-
-	if (flags & DC_EXEC) {
-		v->colourmap = PAL_NONE; // invalidate vehicle colour map
-		SetWindowDirty(WC_VEHICLE_DETAILS, v->index);
-		SetWindowDirty(WC_VEHICLE_DEPOT, v->tile);
-		InvalidateWindowClassesData(WC_SHIPS_LIST, 0);
-	}
-	v->InvalidateNewGRFCacheOfChain(); // always invalidate; querycost might have filled it
-
-	return cost;
-
-}
