@@ -1,4 +1,4 @@
-/* $Id: station_gui.cpp 19995 2010-06-19 16:35:37Z rubidium $ */
+/* $Id: station_gui.cpp 22601 2011-06-18 19:43:51Z rubidium $ */
 
 /*
  * This file is part of OpenTTD.
@@ -28,9 +28,9 @@
 #include "company_base.h"
 #include "sortlist_type.h"
 #include "core/geometry_func.hpp"
+#include "vehiclelist.h"
 
 #include "table/strings.h"
-#include "table/sprites.h"
 
 /**
  * Draw a (multi)line of cargos seperated by commas, and prefixed with a string.
@@ -45,18 +45,18 @@ static int DrawCargoListText(uint32 cargo_mask, const Rect &r, StringID prefix)
 	char string[512];
 	char *b = string;
 
-	for (CargoID i = 0; i < NUM_CARGO; i++) {
+	CargoID i;
+	FOR_EACH_SET_CARGO_ID(i, cargo_mask) {
 		if (b >= lastof(string) - (1 + 2 * 4)) break; // ',' or ' ' and two calls to Utf8Encode()
-		if (HasBit(cargo_mask, i)) {
-			if (first) {
-				first = false;
-			} else {
-				/* Add a comma if this is not the first item */
-				*b++ = ',';
-				*b++ = ' ';
-			}
-			b = InlineString(b, CargoSpec::Get(i)->name);
+
+		if (first) {
+			first = false;
+		} else {
+			/* Add a comma if this is not the first item */
+			*b++ = ',';
+			*b++ = ' ';
 		}
+		b = InlineString(b, CargoSpec::Get(i)->name);
 	}
 
 	/* If first is still true then no cargo is accepted */
@@ -84,7 +84,8 @@ static int DrawCargoListText(uint32 cargo_mask, const Rect &r, StringID prefix)
 int DrawStationCoverageAreaText(int left, int right, int top, StationCoverageType sct, int rad, bool supplies)
 {
 	TileIndex tile = TileVirtXY(_thd.pos.x, _thd.pos.y);
-	if (tile < MapSize()) {
+	uint32 cargo_mask = 0;
+	if (_thd.drawstyle == HT_RECT && tile < MapSize()) {
 		CargoArray cargos;
 		if (supplies) {
 			cargos = GetProductionAroundTiles(tile, _thd.size.x / TILE_SIZE, _thd.size.y / TILE_SIZE, rad);
@@ -93,7 +94,6 @@ int DrawStationCoverageAreaText(int left, int right, int top, StationCoverageTyp
 		}
 
 		/* Convert cargo counts to a set of cargo bits, and draw the result. */
-		uint32 cargo_mask = 0;
 		for (CargoID i = 0; i < NUM_CARGO; i++) {
 			switch (sct) {
 				case SCT_PASSENGERS_ONLY: if (!IsCargoInClass(i, CC_PASSENGERS)) continue; break;
@@ -103,11 +103,9 @@ int DrawStationCoverageAreaText(int left, int right, int top, StationCoverageTyp
 			}
 			if (cargos[i] >= (supplies ? 1U : 8U)) SetBit(cargo_mask, i);
 		}
-		Rect r = {left, top, right, INT32_MAX};
-		return DrawCargoListText(cargo_mask, r, supplies ? STR_STATION_BUILD_SUPPLIES_CARGO : STR_STATION_BUILD_ACCEPTS_CARGO);
 	}
-
-	return top;
+	Rect r = {left, top, right, INT32_MAX};
+	return DrawCargoListText(cargo_mask, r, supplies ? STR_STATION_BUILD_SUPPLIES_CARGO : STR_STATION_BUILD_ACCEPTS_CARGO);
 }
 
 /**
@@ -181,6 +179,7 @@ enum StationListWidgets {
 	SLW_LIST,           ///< The main panel, list of stations
 	SLW_SCROLLBAR,      ///< Scrollbar next to the main panel
 
+	/* Vehicletypes need to be in order of StationFacility due to bit magic */
 	SLW_TRAIN,          ///< 'TRAIN' button - list only facilities where is a railroad station
 	SLW_TRUCK,          ///< 'TRUCK' button - list only facilities where is a truck stop
 	SLW_BUS,            ///< 'BUS' button - list only facilities where is a bus stop
@@ -216,7 +215,7 @@ protected:
 	static GUIStationList::SortFunction * const sorter_funcs[];
 
 	GUIStationList stations;
-
+	Scrollbar *vscroll;
 
 	/**
 	 * (Re)Build station list
@@ -233,11 +232,11 @@ protected:
 
 		const Station *st;
 		FOR_ALL_STATIONS(st) {
-			if (st->owner == owner || (st->owner == OWNER_NONE && HasStationInUse(st->index, owner))) {
+			if (st->owner == owner || (st->owner == OWNER_NONE && HasStationInUse(st->index, true, owner))) {
 				if (this->facilities & st->facilities) { // only stations with selected facilities
 					int num_waiting_cargo = 0;
 					for (CargoID j = 0; j < NUM_CARGO; j++) {
-						if (!st->goods[j].cargo.Empty()) {
+						if (HasBit(st->goods[j].acceptance_pickup, GoodsEntry::PICKUP)) {
 							num_waiting_cargo++; // count number of waiting cargo
 							if (HasBit(this->cargo_filter, j)) {
 								*this->stations.Append() = st;
@@ -256,7 +255,7 @@ protected:
 		this->stations.Compact();
 		this->stations.RebuildDone();
 
-		this->vscroll.SetCount(this->stations.Length()); // Update the scrollbar
+		this->vscroll->SetCount(this->stations.Length()); // Update the scrollbar
 	}
 
 	/** Sort stations by their name */
@@ -288,8 +287,8 @@ protected:
 	{
 		Money diff = 0;
 
-		for (CargoID j = 0; j < NUM_CARGO; j++) {
-			if (!HasBit(cargo_filter, j)) continue;
+		CargoID j;
+		FOR_EACH_SET_CARGO_ID(j, cargo_filter) {
 			if (!(*a)->goods[j].cargo.Empty()) diff += GetTransportedGoodsIncome((*a)->goods[j].cargo.Count(), 20, 50, j);
 			if (!(*b)->goods[j].cargo.Empty()) diff -= GetTransportedGoodsIncome((*b)->goods[j].cargo.Count(), 20, 50, j);
 		}
@@ -303,8 +302,8 @@ protected:
 		byte maxr1 = 0;
 		byte maxr2 = 0;
 
-		for (CargoID j = 0; j < NUM_CARGO; j++) {
-			if (!HasBit(cargo_filter, j)) continue;
+		CargoID j;
+		FOR_EACH_SET_CARGO_ID(j, cargo_filter) {
 			if (HasBit((*a)->goods[j].acceptance_pickup, GoodsEntry::PICKUP)) maxr1 = max(maxr1, (*a)->goods[j].rating);
 			if (HasBit((*b)->goods[j].acceptance_pickup, GoodsEntry::PICKUP)) maxr2 = max(maxr2, (*b)->goods[j].rating);
 		}
@@ -348,12 +347,14 @@ public:
 		this->stations.NeedResort();
 		this->SortStationsList();
 
-		this->InitNested(desc, window_number);
+		this->CreateNestedTree(desc);
+		this->vscroll = this->GetScrollbar(SLW_SCROLLBAR);
+		this->FinishInitNested(desc, window_number);
 		this->owner = (Owner)this->window_number;
 
-		for (uint i = 0; i < NUM_CARGO; i++) {
-			const CargoSpec *cs = CargoSpec::Get(i);
-			if (cs->IsValid() && HasBit(this->cargo_filter, i)) this->LowerWidget(SLW_CARGOSTART + i);
+		CargoID cid;
+		FOR_EACH_SET_CARGO_ID(cid, this->cargo_filter) {
+			if (CargoSpec::Get(cid)->IsValid()) this->LowerWidget(SLW_CARGOSTART + cid);
 		}
 
 		if (this->cargo_filter == this->cargo_filter_max) this->cargo_filter = _cargo_mask;
@@ -361,8 +362,6 @@ public:
 		for (uint i = 0; i < 5; i++) {
 			if (HasBit(this->facilities, i)) this->LowerWidget(i + SLW_TRAIN);
 		}
-		this->SetWidgetLoweredState(SLW_FACILALL, this->facilities == (FACIL_TRAIN | FACIL_TRUCK_STOP | FACIL_BUS_STOP | FACIL_AIRPORT | FACIL_DOCK));
-		this->SetWidgetLoweredState(SLW_CARGOALL, this->cargo_filter == _cargo_mask && this->include_empty);
 		this->SetWidgetLoweredState(SLW_NOCARGOWAITING, this->include_empty);
 
 		this->GetWidget<NWidgetCore>(SLW_SORTDROPBTN)->widget_data = this->sorter_names[this->stations.SortType()];
@@ -378,7 +377,7 @@ public:
 		switch (widget) {
 			case SLW_SORTBY: {
 				Dimension d = GetStringBoundingBox(this->GetWidget<NWidgetCore>(widget)->widget_data);
-				d.width += padding.width + WD_SORTBUTTON_ARROW_WIDTH * 2; // Doubled since the word is centered, also looks nice.
+				d.width += padding.width + WD_SORTBUTTON_ARROW_WIDTH * 2; // Doubled since the string is centred and it also looks better.
 				d.height += padding.height;
 				*size = maxdim(*size, d);
 				break;
@@ -449,10 +448,10 @@ public:
 				break;
 
 			case SLW_LIST: {
-				bool rtl = _dynlang.text_dir == TD_RTL;
-				int max = min(this->vscroll.GetPosition() + this->vscroll.GetCapacity(), this->stations.Length());
+				bool rtl = _current_text_dir == TD_RTL;
+				int max = min(this->vscroll->GetPosition() + this->vscroll->GetCapacity(), this->stations.Length());
 				int y = r.top + WD_FRAMERECT_TOP;
-				for (int i = this->vscroll.GetPosition(); i < max; ++i) { // do until max number of stations of owner
+				for (int i = this->vscroll->GetPosition(); i < max; ++i) { // do until max number of stations of owner
 					const Station *st = this->stations[i];
 					assert(st->xy != INVALID_TILE);
 
@@ -486,7 +485,7 @@ public:
 					y += FONT_HEIGHT_NORMAL;
 				}
 
-				if (this->vscroll.GetCount() == 0) { // company has no stations
+				if (this->vscroll->GetCount() == 0) { // company has no stations
 					DrawString(r.left + WD_FRAMERECT_LEFT, r.right - WD_FRAMERECT_RIGHT, y, STR_STATION_LIST_NONE);
 					return;
 				}
@@ -495,13 +494,13 @@ public:
 
 			case SLW_NOCARGOWAITING: {
 				int cg_ofst = this->IsWidgetLowered(widget) ? 2 : 1;
-				DrawString(r.left + cg_ofst, r.right + cg_ofst, r.top + cg_ofst, STR_ABBREV_NONE, TC_BLACK, SA_CENTER);
+				DrawString(r.left + cg_ofst, r.right + cg_ofst, r.top + cg_ofst, STR_ABBREV_NONE, TC_BLACK, SA_HOR_CENTER);
 				break;
 			}
 
 			case SLW_CARGOALL: {
 				int cg_ofst = this->IsWidgetLowered(widget) ? 2 : 1;
-				DrawString(r.left + cg_ofst, r.right + cg_ofst, r.top + cg_ofst, STR_ABBREV_ALL, TC_BLACK, SA_CENTER);
+				DrawString(r.left + cg_ofst, r.right + cg_ofst, r.top + cg_ofst, STR_ABBREV_ALL, TC_BLACK, SA_HOR_CENTER);
 				break;
 			}
 
@@ -517,7 +516,7 @@ public:
 					if (cs->IsValid()) {
 						int cg_ofst = HasBit(this->cargo_filter, cs->Index()) ? 2 : 1;
 						GfxFillRect(r.left + cg_ofst, r.top + cg_ofst, r.right - 2 + cg_ofst, r.bottom - 2 + cg_ofst, cs->rating_colour);
-						DrawString(r.left + cg_ofst, r.right + cg_ofst, r.top + cg_ofst, cs->abbrev, TC_BLACK, SA_CENTER);
+						DrawString(r.left + cg_ofst, r.right + cg_ofst, r.top + cg_ofst, cs->abbrev, TC_BLACK, SA_HOR_CENTER);
 					}
 				}
 				break;
@@ -528,7 +527,7 @@ public:
 	{
 		if (widget == SLW_CAPTION) {
 			SetDParam(0, this->window_number);
-			SetDParam(1, this->vscroll.GetCount());
+			SetDParam(1, this->vscroll->GetCount());
 		}
 	}
 
@@ -536,12 +535,7 @@ public:
 	{
 		switch (widget) {
 			case SLW_LIST: {
-				uint32 id_v = (pt.y - this->GetWidget<NWidgetBase>(SLW_LIST)->pos_y - WD_FRAMERECT_TOP) / FONT_HEIGHT_NORMAL;
-
-				if (id_v >= this->vscroll.GetCapacity()) return; // click out of bounds
-
-				id_v += this->vscroll.GetPosition();
-
+				uint id_v = this->vscroll->GetScrolledRowFromWidget(pt.y, this, SLW_LIST, 0, FONT_HEIGHT_NORMAL);
 				if (id_v >= this->stations.Length()) return; // click out of list bound
 
 				const Station *st = this->stations[id_v];
@@ -569,19 +563,17 @@ public:
 					FOR_EACH_SET_BIT(i, this->facilities) {
 						this->RaiseWidget(i + SLW_TRAIN);
 					}
-					SetBit(this->facilities, widget - SLW_TRAIN);
+					this->facilities = 1 << (widget - SLW_TRAIN);
 					this->LowerWidget(widget);
 				}
-				this->SetWidgetLoweredState(SLW_FACILALL, this->facilities == (FACIL_TRAIN | FACIL_TRUCK_STOP | FACIL_BUS_STOP | FACIL_AIRPORT | FACIL_DOCK));
 				this->stations.ForceRebuild();
 				this->SetDirty();
 				break;
 
 			case SLW_FACILALL:
-				for (uint i = 0; i < 5; i++) {
-					this->LowerWidget(i + SLW_TRAIN);
+				for (uint i = SLW_TRAIN; i <= SLW_SHIP; i++) {
+					this->LowerWidget(i);
 				}
-				this->LowerWidget(SLW_FACILALL);
 
 				this->facilities = FACIL_TRAIN | FACIL_TRUCK_STOP | FACIL_BUS_STOP | FACIL_AIRPORT | FACIL_DOCK;
 				this->stations.ForceRebuild();
@@ -594,7 +586,6 @@ public:
 					if (cs->IsValid()) this->LowerWidget(SLW_CARGOSTART + i);
 				}
 				this->LowerWidget(SLW_NOCARGOWAITING);
-				this->LowerWidget(SLW_CARGOALL);
 
 				this->cargo_filter = _cargo_mask;
 				this->include_empty = true;
@@ -629,7 +620,6 @@ public:
 
 					this->LowerWidget(SLW_NOCARGOWAITING);
 				}
-				this->SetWidgetLoweredState(SLW_CARGOALL, this->cargo_filter == _cargo_mask && this->include_empty);
 				this->stations.ForceRebuild();
 				this->SetDirty();
 				break;
@@ -656,7 +646,6 @@ public:
 						SetBit(this->cargo_filter, cs->Index());
 						this->LowerWidget(widget);
 					}
-					this->SetWidgetLoweredState(SLW_CARGOALL, this->cargo_filter == _cargo_mask && this->include_empty);
 					this->stations.ForceRebuild();
 					this->SetDirty();
 				}
@@ -693,12 +682,18 @@ public:
 
 	virtual void OnResize()
 	{
-		this->vscroll.SetCapacityFromWidget(this, SLW_LIST, WD_FRAMERECT_TOP + WD_FRAMERECT_BOTTOM);
+		this->vscroll->SetCapacityFromWidget(this, SLW_LIST, WD_FRAMERECT_TOP + WD_FRAMERECT_BOTTOM);
 	}
 
-	virtual void OnInvalidateData(int data)
+	/**
+	 * Some data on this window has become invalid.
+	 * @param data Information about the changed data.
+	 * @param gui_scope Whether the call is done from GUI scope. You may not do everything when not in GUI scope. See #InvalidateWindowData() for details.
+	 */
+	virtual void OnInvalidateData(int data = 0, bool gui_scope = true)
 	{
 		if (data == 0) {
+			/* This needs to be done in command-scope to enforce rebuilding before resorting invalid data */
 			this->stations.ForceRebuild();
 		} else {
 			this->stations.ForceResort();
@@ -732,7 +727,8 @@ const StringID CompanyStationsWindow::sorter_names[] = {
 	INVALID_STRING_ID
 };
 
-/** Make a horizontal row of cargo buttons, starting at widget #SLW_CARGOSTART.
+/**
+ * Make a horizontal row of cargo buttons, starting at widget #SLW_CARGOSTART.
  * @param biggest_index Pointer to store biggest used widget number of the buttons.
  * @return Horizontal row.
  */
@@ -772,13 +768,13 @@ static const NWidgetPart _nested_company_stations_widgets[] = {
 		NWidget(WWT_TEXTBTN, COLOUR_GREY, SLW_TRAIN), SetMinimalSize(14, 11), SetDataTip(STR_TRAIN, STR_STATION_LIST_USE_CTRL_TO_SELECT_MORE), SetFill(0, 1),
 		NWidget(WWT_TEXTBTN, COLOUR_GREY, SLW_TRUCK), SetMinimalSize(14, 11), SetDataTip(STR_LORRY, STR_STATION_LIST_USE_CTRL_TO_SELECT_MORE), SetFill(0, 1),
 		NWidget(WWT_TEXTBTN, COLOUR_GREY, SLW_BUS), SetMinimalSize(14, 11), SetDataTip(STR_BUS, STR_STATION_LIST_USE_CTRL_TO_SELECT_MORE), SetFill(0, 1),
-		NWidget(WWT_TEXTBTN, COLOUR_GREY, SLW_AIRPLANE), SetMinimalSize(14, 11), SetDataTip(STR_PLANE, STR_STATION_LIST_USE_CTRL_TO_SELECT_MORE), SetFill(0, 1),
 		NWidget(WWT_TEXTBTN, COLOUR_GREY, SLW_SHIP), SetMinimalSize(14, 11), SetDataTip(STR_SHIP, STR_STATION_LIST_USE_CTRL_TO_SELECT_MORE), SetFill(0, 1),
-		NWidget(WWT_PANEL, COLOUR_GREY, SLW_FACILALL), SetMinimalSize(14, 11), SetDataTip(0x0, STR_STATION_LIST_SELECT_ALL_FACILITIES), SetFill(0, 1), EndContainer(),
+		NWidget(WWT_TEXTBTN, COLOUR_GREY, SLW_AIRPLANE), SetMinimalSize(14, 11), SetDataTip(STR_PLANE, STR_STATION_LIST_USE_CTRL_TO_SELECT_MORE), SetFill(0, 1),
+		NWidget(WWT_PUSHBTN, COLOUR_GREY, SLW_FACILALL), SetMinimalSize(14, 11), SetDataTip(0x0, STR_STATION_LIST_SELECT_ALL_FACILITIES), SetFill(0, 1),
 		NWidget(WWT_PANEL, COLOUR_GREY), SetMinimalSize(5, 11), SetFill(0, 1), EndContainer(),
 		NWidgetFunction(CargoWidgets),
 		NWidget(WWT_PANEL, COLOUR_GREY, SLW_NOCARGOWAITING), SetMinimalSize(14, 11), SetDataTip(0x0, STR_STATION_LIST_NO_WAITING_CARGO), SetFill(0, 1), EndContainer(),
-		NWidget(WWT_PANEL, COLOUR_GREY, SLW_CARGOALL), SetMinimalSize(14, 11), SetDataTip(0x0, STR_STATION_LIST_SELECT_ALL_TYPES), SetFill(0, 1), EndContainer(),
+		NWidget(WWT_PUSHBTN, COLOUR_GREY, SLW_CARGOALL), SetMinimalSize(14, 11), SetDataTip(0x0, STR_STATION_LIST_SELECT_ALL_TYPES), SetFill(0, 1),
 		NWidget(WWT_PANEL, COLOUR_GREY), SetDataTip(0x0, STR_NULL), SetResize(1, 0), SetFill(1, 1), EndContainer(),
 	EndContainer(),
 	NWidget(NWID_HORIZONTAL),
@@ -787,9 +783,9 @@ static const NWidgetPart _nested_company_stations_widgets[] = {
 		NWidget(WWT_PANEL, COLOUR_GREY), SetDataTip(0x0, STR_NULL), SetResize(1, 0), SetFill(1, 1), EndContainer(),
 	EndContainer(),
 	NWidget(NWID_HORIZONTAL),
-		NWidget(WWT_PANEL, COLOUR_GREY, SLW_LIST), SetMinimalSize(346, 125), SetResize(1, 10), SetDataTip(0x0, STR_STATION_LIST_TOOLTIP), EndContainer(),
+		NWidget(WWT_PANEL, COLOUR_GREY, SLW_LIST), SetMinimalSize(346, 125), SetResize(1, 10), SetDataTip(0x0, STR_STATION_LIST_TOOLTIP), SetScrollbar(SLW_SCROLLBAR), EndContainer(),
 		NWidget(NWID_VERTICAL),
-			NWidget(WWT_SCROLLBAR, COLOUR_GREY, SLW_SCROLLBAR),
+			NWidget(NWID_VSCROLLBAR, COLOUR_GREY, SLW_SCROLLBAR),
 			NWidget(WWT_RESIZEBOX, COLOUR_GREY),
 		EndContainer(),
 	EndContainer(),
@@ -798,7 +794,7 @@ static const NWidgetPart _nested_company_stations_widgets[] = {
 static const WindowDesc _company_stations_desc(
 	WDP_AUTO, 358, 162,
 	WC_STATION_LIST, WC_NONE,
-	0,
+	WDF_UNCLICK_BUTTONS,
 	_nested_company_stations_widgets, lengthof(_nested_company_stations_widgets)
 );
 
@@ -822,8 +818,8 @@ static const NWidgetPart _nested_station_view_widgets[] = {
 		NWidget(WWT_STICKYBOX, COLOUR_GREY),
 	EndContainer(),
 	NWidget(NWID_HORIZONTAL),
-		NWidget(WWT_PANEL, COLOUR_GREY, SVW_WAITING), SetMinimalSize(237, 52), SetResize(1, 10), EndContainer(),
-		NWidget(WWT_SCROLLBAR, COLOUR_GREY, SVW_SCROLLBAR),
+		NWidget(WWT_PANEL, COLOUR_GREY, SVW_WAITING), SetMinimalSize(237, 52), SetResize(1, 10), SetScrollbar(SVW_SCROLLBAR), EndContainer(),
+		NWidget(NWID_VSCROLLBAR, COLOUR_GREY, SVW_SCROLLBAR),
 	EndContainer(),
 	NWidget(WWT_PANEL, COLOUR_GREY, SVW_ACCEPTLIST), SetMinimalSize(249, 32), SetResize(1, 0), EndContainer(),
 	NWidget(NWID_HORIZONTAL, NC_EQUALSIZE),
@@ -835,8 +831,8 @@ static const NWidgetPart _nested_station_view_widgets[] = {
 				SetDataTip(STR_BUTTON_RENAME, STR_STATION_VIEW_RENAME_TOOLTIP),
 		NWidget(WWT_PUSHTXTBTN, COLOUR_GREY, SVW_TRAINS), SetMinimalSize(14, 12), SetFill(0, 1), SetDataTip(STR_TRAIN, STR_STATION_VIEW_SCHEDULED_TRAINS_TOOLTIP),
 		NWidget(WWT_PUSHTXTBTN, COLOUR_GREY, SVW_ROADVEHS), SetMinimalSize(14, 12), SetFill(0, 1), SetDataTip(STR_LORRY, STR_STATION_VIEW_SCHEDULED_ROAD_VEHICLES_TOOLTIP),
-		NWidget(WWT_PUSHTXTBTN, COLOUR_GREY, SVW_PLANES),  SetMinimalSize(14, 12), SetFill(0, 1), SetDataTip(STR_PLANE, STR_STATION_VIEW_SCHEDULED_AIRCRAFT_TOOLTIP),
 		NWidget(WWT_PUSHTXTBTN, COLOUR_GREY, SVW_SHIPS), SetMinimalSize(14, 12), SetFill(0, 1), SetDataTip(STR_SHIP, STR_STATION_VIEW_SCHEDULED_SHIPS_TOOLTIP),
+		NWidget(WWT_PUSHTXTBTN, COLOUR_GREY, SVW_PLANES),  SetMinimalSize(14, 12), SetFill(0, 1), SetDataTip(STR_PLANE, STR_STATION_VIEW_SCHEDULED_AIRCRAFT_TOOLTIP),
 		NWidget(WWT_RESIZEBOX, COLOUR_GREY),
 	EndContainer(),
 };
@@ -858,7 +854,7 @@ static void DrawCargoIcons(CargoID i, uint waiting, int left, int right, int y)
 
 	SpriteID sprite = CargoSpec::Get(i)->GetCargoIcon();
 
-	int x = _dynlang.text_dir == TD_RTL ? right - num * 10 : left;
+	int x = _current_text_dir == TD_RTL ? right - num * 10 : left;
 	do {
 		DrawSprite(sprite, PAL_NONE, x, y);
 		x += 10;
@@ -888,6 +884,7 @@ struct StationViewWindow : public Window {
 	uint expand_shrink_width;     ///< The width allocated to the expand/shrink 'button'
 	int rating_lines;             ///< Number of lines in the cargo ratings view.
 	int accepts_lines;            ///< Number of lines in the accepted cargo view.
+	Scrollbar *vscroll;
 
 	/** Height of the #SVW_ACCEPTLIST widget for different views. */
 	enum AcceptListHeight {
@@ -901,6 +898,7 @@ struct StationViewWindow : public Window {
 		this->accepts_lines = ALH_ACCEPTS;
 
 		this->CreateNestedTree(desc);
+		this->vscroll = this->GetScrollbar(SVW_SCROLLBAR);
 		/* Nested widget tree creation is done in two steps to ensure that this->GetWidget<NWidgetCore>(SVW_ACCEPTS) exists in UpdateWidgetSize(). */
 		this->FinishInitNested(desc, window_number);
 
@@ -910,12 +908,13 @@ struct StationViewWindow : public Window {
 
 	~StationViewWindow()
 	{
-		WindowNumber wno = (this->window_number << 16) | VLW_STATION_LIST | Station::Get(this->window_number)->owner;
-
-		DeleteWindowById(WC_TRAINS_LIST, wno | (VEH_TRAIN << 11), false);
-		DeleteWindowById(WC_ROADVEH_LIST, wno | (VEH_ROAD << 11), false);
-		DeleteWindowById(WC_SHIPS_LIST, wno | (VEH_SHIP << 11), false);
-		DeleteWindowById(WC_AIRCRAFT_LIST, wno | (VEH_AIRCRAFT << 11), false);
+		Owner owner = Station::Get(this->window_number)->owner;
+		if (!Company::IsValidID(owner)) owner = _local_company;
+		if (!Company::IsValidID(owner)) return; // Spectators
+		DeleteWindowById(WC_TRAINS_LIST,   VehicleListIdentifier(VL_STATION_LIST, VEH_TRAIN,    owner, this->window_number).Pack(), false);
+		DeleteWindowById(WC_ROADVEH_LIST,  VehicleListIdentifier(VL_STATION_LIST, VEH_ROAD,     owner, this->window_number).Pack(), false);
+		DeleteWindowById(WC_SHIPS_LIST,    VehicleListIdentifier(VL_STATION_LIST, VEH_SHIP,     owner, this->window_number).Pack(), false);
+		DeleteWindowById(WC_AIRCRAFT_LIST, VehicleListIdentifier(VL_STATION_LIST, VEH_AIRCRAFT, owner, this->window_number).Pack(), false);
 	}
 
 	virtual void UpdateWidgetSize(int widget, Dimension *size, const Dimension &padding, Dimension *fill, Dimension *resize)
@@ -939,15 +938,15 @@ struct StationViewWindow : public Window {
 		uint32 transfers = 0;
 		this->OrderWaitingCargo(&cargolist, &transfers);
 
-		this->vscroll.SetCount((int)cargolist.size() + 1); // update scrollbar
+		this->vscroll->SetCount((int)cargolist.size() + 1); // update scrollbar
 
 		/* disable some buttons */
 		const Station *st = Station::Get(this->window_number);
 		this->SetWidgetDisabledState(SVW_RENAME,   st->owner != _local_company);
 		this->SetWidgetDisabledState(SVW_TRAINS,   !(st->facilities & FACIL_TRAIN));
 		this->SetWidgetDisabledState(SVW_ROADVEHS, !(st->facilities & FACIL_TRUCK_STOP) && !(st->facilities & FACIL_BUS_STOP));
-		this->SetWidgetDisabledState(SVW_PLANES,   !(st->facilities & FACIL_AIRPORT));
 		this->SetWidgetDisabledState(SVW_SHIPS,    !(st->facilities & FACIL_DOCK));
+		this->SetWidgetDisabledState(SVW_PLANES,   !(st->facilities & FACIL_AIRPORT));
 
 		this->DrawWidgets();
 
@@ -987,7 +986,8 @@ struct StationViewWindow : public Window {
 		}
 	}
 
-	/** Order waiting cargo by type and destination.
+	/**
+	 * Order waiting cargo by type and destination.
 	 * @param cargolist [out] Ordered cargo.
 	 * @param transfers [out] Bitmask for cargoes being transfered.
 	 * @pre \c *cargolist must be empty.
@@ -1041,7 +1041,8 @@ struct StationViewWindow : public Window {
 		}
 	}
 
-	/** Draw waiting cargo.
+	/**
+	 * Draw waiting cargo.
 	 * @param r Rectangle of the widget.
 	 * @param cargolist Cargo, ordered by type and destination.
 	 * @param transfers Bitmask for cargoes that are transfered.
@@ -1049,7 +1050,7 @@ struct StationViewWindow : public Window {
 	void DrawWaitingCargo(const Rect &r, const CargoDataList &cargolist, uint32 transfers) const
 	{
 		int y = r.top + WD_FRAMERECT_TOP;
-		int pos = this->vscroll.GetPosition();
+		int pos = this->vscroll->GetPosition();
 
 		const Station *st = Station::Get(this->window_number);
 		if (--pos < 0) {
@@ -1062,14 +1063,14 @@ struct StationViewWindow : public Window {
 			y += FONT_HEIGHT_NORMAL;
 		}
 
-		bool rtl = _dynlang.text_dir == TD_RTL;
+		bool rtl = _current_text_dir == TD_RTL;
 		int text_left    = rtl ? r.left + this->expand_shrink_width : r.left + WD_FRAMERECT_LEFT;
 		int text_right   = rtl ? r.right - WD_FRAMERECT_LEFT : r.right - this->expand_shrink_width;
 		int shrink_left  = rtl ? r.left + WD_FRAMERECT_LEFT : r.right - this->expand_shrink_width + WD_FRAMERECT_LEFT;
 		int shrink_right = rtl ? r.left + this->expand_shrink_width - WD_FRAMERECT_RIGHT : r.right - WD_FRAMERECT_RIGHT;
 
 
-		int maxrows = this->vscroll.GetCapacity();
+		int maxrows = this->vscroll->GetCapacity();
 		for (CargoDataList::const_iterator it = cargolist.begin(); it != cargolist.end() && pos > -maxrows; ++it) {
 			if (--pos < 0) {
 				const CargoData *cd = &(*it);
@@ -1098,7 +1099,8 @@ struct StationViewWindow : public Window {
 		}
 	}
 
-	/** Draw accepted cargo in the #SVW_ACCEPTLIST widget.
+	/**
+	 * Draw accepted cargo in the #SVW_ACCEPTLIST widget.
 	 * @param r Rectangle of the widget.
 	 * @return Number of lines needed for drawing the accepted cargo.
 	 */
@@ -1112,10 +1114,11 @@ struct StationViewWindow : public Window {
 		}
 		Rect s = {r.left + WD_FRAMERECT_LEFT, r.top + WD_FRAMERECT_TOP, r.right - WD_FRAMERECT_RIGHT, INT32_MAX};
 		int bottom = DrawCargoListText(cargo_mask, s, STR_STATION_VIEW_ACCEPTS_CARGO);
-		return (bottom - r.top - WD_FRAMERECT_TOP + FONT_HEIGHT_NORMAL - 1) / FONT_HEIGHT_NORMAL;
+		return CeilDiv(bottom - r.top - WD_FRAMERECT_TOP, FONT_HEIGHT_NORMAL);
 	}
 
-	/** Draw cargo ratings in the #SVW_ACCEPTLIST widget.
+	/**
+	 * Draw cargo ratings in the #SVW_ACCEPTLIST widget.
 	 * @param r Rectangle of the widget.
 	 * @return Number of lines needed for drawing the cargo ratings.
 	 */
@@ -1128,7 +1131,7 @@ struct StationViewWindow : public Window {
 		y += FONT_HEIGHT_NORMAL;
 
 		const CargoSpec *cs;
-		FOR_ALL_CARGOSPECS(cs) {
+		FOR_ALL_SORTED_STANDARD_CARGOSPECS(cs) {
 			const GoodsEntry *ge = &st->goods[cs->Index()];
 			if (!HasBit(ge->acceptance_pickup, GoodsEntry::PICKUP)) continue;
 
@@ -1138,7 +1141,7 @@ struct StationViewWindow : public Window {
 			DrawString(r.left + WD_FRAMERECT_LEFT + 6, r.right - WD_FRAMERECT_RIGHT - 6, y, STR_STATION_VIEW_CARGO_RATING);
 			y += FONT_HEIGHT_NORMAL;
 		}
-		return (y - r.top - WD_FRAMERECT_TOP + FONT_HEIGHT_NORMAL - 1) / FONT_HEIGHT_NORMAL;
+		return CeilDiv(y - r.top - WD_FRAMERECT_TOP, FONT_HEIGHT_NORMAL);
 	}
 
 	void HandleCargoWaitingClick(int row)
@@ -1158,7 +1161,7 @@ struct StationViewWindow : public Window {
 	{
 		switch (widget) {
 			case SVW_WAITING:
-				this->HandleCargoWaitingClick((pt.y - this->GetWidget<NWidgetBase>(SVW_WAITING)->pos_y - WD_FRAMERECT_TOP) / FONT_HEIGHT_NORMAL + this->vscroll.GetPosition());
+				this->HandleCargoWaitingClick(this->vscroll->GetScrolledRowFromWidget(pt.y, this, SVW_WAITING, WD_FRAMERECT_TOP, FONT_HEIGHT_NORMAL));
 				break;
 
 			case SVW_LOCATION:
@@ -1186,14 +1189,14 @@ struct StationViewWindow : public Window {
 
 			case SVW_RENAME:
 				SetDParam(0, this->window_number);
-				ShowQueryString(STR_STATION_NAME, STR_STATION_VIEW_RENAME_STATION_CAPTION, MAX_LENGTH_STATION_NAME_BYTES, MAX_LENGTH_STATION_NAME_PIXELS,
-						this, CS_ALPHANUMERAL, QSF_ENABLE_DEFAULT);
+				ShowQueryString(STR_STATION_NAME, STR_STATION_VIEW_RENAME_STATION_CAPTION, MAX_LENGTH_STATION_NAME_CHARS, MAX_LENGTH_STATION_NAME_PIXELS,
+						this, CS_ALPHANUMERAL, QSF_ENABLE_DEFAULT | QSF_LEN_IN_CHARS);
 				break;
 
 			case SVW_TRAINS:   // Show list of scheduled trains to this station
 			case SVW_ROADVEHS: // Show list of scheduled road-vehicles to this station
-			case SVW_PLANES:   // Show list of scheduled aircraft to this station
 			case SVW_SHIPS:    // Show list of scheduled ships to this station
+			case SVW_PLANES:   // Show list of scheduled aircraft to this station
 				ShowVehicleListWindow(this->owner, (VehicleType)(widget - SVW_TRAINS), (StationID)this->window_number);
 				break;
 		}
@@ -1208,7 +1211,7 @@ struct StationViewWindow : public Window {
 
 	virtual void OnResize()
 	{
-		this->vscroll.SetCapacityFromWidget(this, SVW_WAITING, WD_FRAMERECT_TOP + WD_FRAMERECT_BOTTOM);
+		this->vscroll->SetCapacityFromWidget(this, SVW_WAITING, WD_FRAMERECT_TOP + WD_FRAMERECT_BOTTOM);
 	}
 };
 
@@ -1272,7 +1275,7 @@ static bool AddNearbyStation(TileIndex tile, void *user_data)
 	T *st = T::Get(sid);
 	if (st->owner != _local_company || _stations_nearby_list.Contains(sid)) return false;
 
-	if (st->rect.BeforeAddRect(ctx->tile, ctx->w, ctx->h, StationRect::ADD_TEST)) {
+	if (st->rect.BeforeAddRect(ctx->tile, ctx->w, ctx->h, StationRect::ADD_TEST).Succeeded()) {
 		*_stations_nearby_list.Append() = sid;
 	}
 
@@ -1287,7 +1290,7 @@ static bool AddNearbyStation(TileIndex tile, void *user_data)
  * @param distant_join Search for adjacent stations (false) or stations fully
  *                     within station spread
  * @tparam T the type of station to look for
- **/
+ */
 template <class T>
 static const T *FindStationsNearby(TileArea ta, bool distant_join)
 {
@@ -1344,9 +1347,9 @@ static const NWidgetPart _nested_select_station_widgets[] = {
 		NWidget(WWT_CAPTION, COLOUR_DARK_GREEN, JSW_WIDGET_CAPTION), SetDataTip(STR_JOIN_STATION_CAPTION, STR_TOOLTIP_WINDOW_TITLE_DRAG_THIS),
 	EndContainer(),
 	NWidget(NWID_HORIZONTAL),
-		NWidget(WWT_PANEL, COLOUR_DARK_GREEN, JSW_PANEL), SetResize(1, 0), EndContainer(),
+		NWidget(WWT_PANEL, COLOUR_DARK_GREEN, JSW_PANEL), SetResize(1, 0), SetScrollbar(JSW_SCROLLBAR), EndContainer(),
 		NWidget(NWID_VERTICAL),
-			NWidget(WWT_SCROLLBAR, COLOUR_DARK_GREEN, JSW_SCROLLBAR),
+			NWidget(NWID_VSCROLLBAR, COLOUR_DARK_GREEN, JSW_SCROLLBAR),
 			NWidget(WWT_RESIZEBOX, COLOUR_DARK_GREEN),
 		EndContainer(),
 	EndContainer(),
@@ -1360,6 +1363,7 @@ template <class T>
 struct SelectStationWindow : Window {
 	CommandContainer select_station_cmd; ///< Command to build new station
 	TileArea area; ///< Location of new station
+	Scrollbar *vscroll;
 
 	SelectStationWindow(const WindowDesc *desc, CommandContainer cmd, TileArea ta) :
 		Window(),
@@ -1367,6 +1371,7 @@ struct SelectStationWindow : Window {
 		area(ta)
 	{
 		this->CreateNestedTree(desc);
+		this->vscroll = this->GetScrollbar(JSW_SCROLLBAR);
 		this->GetWidget<NWidgetCore>(JSW_WIDGET_CAPTION)->widget_data = T::EXPECTED_FACIL == FACIL_WAYPOINT ? STR_JOIN_WAYPOINT_CAPTION : STR_JOIN_STATION_CAPTION;
 		this->FinishInitNested(desc, 0);
 		this->OnInvalidateData(0);
@@ -1392,24 +1397,19 @@ struct SelectStationWindow : Window {
 		*size = d;
 	}
 
-	virtual void OnPaint()
-	{
-		this->DrawWidgets();
-	}
-
 	virtual void DrawWidget(const Rect &r, int widget) const
 	{
 		if (widget != JSW_PANEL) return;
 
 		uint y = r.top + WD_FRAMERECT_TOP;
-		if (this->vscroll.GetPosition() == 0) {
+		if (this->vscroll->GetPosition() == 0) {
 			DrawString(r.left + WD_FRAMERECT_LEFT, r.right - WD_FRAMERECT_RIGHT, y, T::EXPECTED_FACIL == FACIL_WAYPOINT ? STR_JOIN_WAYPOINT_CREATE_SPLITTED_WAYPOINT : STR_JOIN_STATION_CREATE_SPLITTED_STATION);
 			y += this->resize.step_height;
 		}
 
-		for (uint i = max<uint>(1, this->vscroll.GetPosition()); i <= _stations_nearby_list.Length(); ++i, y += this->resize.step_height) {
+		for (uint i = max<uint>(1, this->vscroll->GetPosition()); i <= _stations_nearby_list.Length(); ++i, y += this->resize.step_height) {
 			/* Don't draw anything if it extends past the end of the window. */
-			if (i - this->vscroll.GetPosition() >= this->vscroll.GetCapacity()) break;
+			if (i - this->vscroll->GetPosition() >= this->vscroll->GetCapacity()) break;
 
 			const T *st = T::Get(_stations_nearby_list[i - 1]);
 			SetDParam(0, st->index);
@@ -1422,7 +1422,7 @@ struct SelectStationWindow : Window {
 	{
 		if (widget != JSW_PANEL) return;
 
-		uint32 st_index = (pt.y - this->GetWidget<NWidgetBase>(JSW_PANEL)->pos_y - WD_FRAMERECT_TOP) / this->resize.step_height + this->vscroll.GetPosition();
+		uint st_index = this->vscroll->GetScrolledRowFromWidget(pt.y, this, JSW_PANEL, WD_FRAMERECT_TOP);
 		bool distant_join = (st_index > 0);
 		if (distant_join) st_index--;
 
@@ -1449,13 +1449,19 @@ struct SelectStationWindow : Window {
 
 	virtual void OnResize()
 	{
-		this->vscroll.SetCapacityFromWidget(this, JSW_PANEL, WD_FRAMERECT_TOP + WD_FRAMERECT_BOTTOM);
+		this->vscroll->SetCapacityFromWidget(this, JSW_PANEL, WD_FRAMERECT_TOP + WD_FRAMERECT_BOTTOM);
 	}
 
-	virtual void OnInvalidateData(int data)
+	/**
+	 * Some data on this window has become invalid.
+	 * @param data Information about the changed data.
+	 * @param gui_scope Whether the call is done from GUI scope. You may not do everything when not in GUI scope. See #InvalidateWindowData() for details.
+	 */
+	virtual void OnInvalidateData(int data = 0, bool gui_scope = true)
 	{
+		if (!gui_scope) return;
 		FindStationsNearby<T>(this->area, true);
-		this->vscroll.SetCount(_stations_nearby_list.Length() + 1);
+		this->vscroll->SetCount(_stations_nearby_list.Length() + 1);
 		this->SetDirty();
 	}
 };
@@ -1485,8 +1491,6 @@ static bool StationJoinerNeeded(CommandContainer cmd, TileArea ta)
 	 * return true (i.e. just flash the old window) */
 	Window *selection_window = FindWindowById(WC_SELECT_STATION, 0);
 	if (selection_window != NULL) {
-		if (!_ctrl_pressed) return true;
-
 		/* Abort current distant-join and start new one */
 		delete selection_window;
 		UpdateTileSelection();
@@ -1500,7 +1504,7 @@ static bool StationJoinerNeeded(CommandContainer cmd, TileArea ta)
 
 	/* Test for adjacent station or station below selection.
 	 * If adjacent-stations is disabled and we are building next to a station, do not show the selection window.
-	 * but join the other station immediatelly. */
+	 * but join the other station immediately. */
 	const T *st = FindStationsNearby<T>(ta, false);
 	return st == NULL && (_settings_game.station.adjacent_stations || _stations_nearby_list.Length() == 0);
 }
@@ -1516,7 +1520,6 @@ void ShowSelectBaseStationIfNeeded(CommandContainer cmd, TileArea ta)
 {
 	if (StationJoinerNeeded<T>(cmd, ta)) {
 		if (!_settings_client.gui.persistent_buildingtools) ResetObjectToPlace();
-		if (BringWindowToFrontById(WC_SELECT_STATION, 0)) return;
 		new SelectStationWindow<T>(&_select_station_desc, cmd, ta);
 	} else {
 		DoCommandP(&cmd);
