@@ -1,4 +1,4 @@
-/* $Id: string_func.h 19875 2010-05-21 16:03:29Z rubidium $ */
+/* $Id: string_func.h 22393 2011-04-30 20:50:41Z rubidium $ */
 
 /*
  * This file is part of OpenTTD.
@@ -7,7 +7,8 @@
  * See the GNU General Public License for more details. You should have received a copy of the GNU General Public License along with OpenTTD. If not, see <http://www.gnu.org/licenses/>.
  */
 
-/** @file string_func.h Functions related to low-level strings.
+/**
+ * @file string_func.h Functions related to low-level strings.
  *
  * @note Be aware of "dangerous" string functions; string functions that
  * have behaviour that could easily cause buffer overruns and such:
@@ -28,94 +29,22 @@
 #include "core/bitmath_func.hpp"
 #include "string_type.h"
 
-/**
- * Appends characters from one string to another.
- *
- * Appends the source string to the destination string with respect of the
- * terminating null-character and the maximum size of the destination
- * buffer.
- *
- * @note usage ttd_strlcat(dst, src, lengthof(dst));
- * @note lengthof() applies only to fixed size arrays
- *
- * @param dst The buffer containing the target string
- * @param src The buffer containing the string to append
- * @param size The maximum size of the destination buffer
- */
 void ttd_strlcat(char *dst, const char *src, size_t size);
-
-/**
- * Copies characters from one buffer to another.
- *
- * Copies the source string to the destination buffer with respect of the
- * terminating null-character and the maximum size of the destination
- * buffer.
- *
- * @note usage ttd_strlcpy(dst, src, lengthof(dst));
- * @note lengthof() applies only to fixed size arrays
- *
- * @param dst The destination buffer
- * @param src The buffer containing the string to copy
- * @param size The maximum size of the destination buffer
- */
 void ttd_strlcpy(char *dst, const char *src, size_t size);
 
-/**
- * Appends characters from one string to another.
- *
- * Appends the source string to the destination string with respect of the
- * terminating null-character and and the last pointer to the last element
- * in the destination buffer. If the last pointer is set to NULL no
- * boundary check is performed.
- *
- * @note usage: strecat(dst, src, lastof(dst));
- * @note lastof() applies only to fixed size arrays
- *
- * @param dst The buffer containing the target string
- * @param src The buffer containing the string to append
- * @param last The pointer to the last element of the destination buffer
- * @return The pointer to the terminating null-character in the destination buffer
- */
 char *strecat(char *dst, const char *src, const char *last);
-
-/**
- * Copies characters from one buffer to another.
- *
- * Copies the source string to the destination buffer with respect of the
- * terminating null-character and the last pointer to the last element in
- * the destination buffer. If the last pointer is set to NULL no boundary
- * check is performed.
- *
- * @note usage: strecpy(dst, src, lastof(dst));
- * @note lastof() applies only to fixed size arrays
- *
- * @param dst The destination buffer
- * @param src The buffer containing the string to copy
- * @param last The pointer to the last element of the destination buffer
- * @return The pointer to the terminating null-character in the destination buffer
- */
 char *strecpy(char *dst, const char *src, const char *last);
 
 int CDECL seprintf(char *str, const char *last, const char *format, ...) WARN_FORMAT(3, 4);
 
 char *CDECL str_fmt(const char *str, ...) WARN_FORMAT(1, 2);
 
-/**
- * Scans the string for valid characters and if it finds invalid ones,
- * replaces them with a question mark '?' (if not ignored)
- * @param str the string to validate
- * @param last the last valid character of str
- * @param allow_newlines whether newlines should be allowed or ignored
- * @param ignore whether to ignore or replace with a question mark
- */
 void str_validate(char *str, const char *last, bool allow_newlines = false, bool ignore = false);
-
-/** Scans the string for colour codes and strips them */
 void str_strip_colours(char *str);
-
-/** Convert the given string to lowercase, only works with ASCII! */
 void strtolower(char *str);
 
+bool StrValid(const char *str, const char *last);
+
 /**
  * Check if a string buffer is empty.
  *
@@ -142,16 +71,8 @@ static inline size_t ttd_strnlen(const char *str, size_t maxlen)
 	return t - str;
 }
 
-/** Convert the md5sum number to a 'hexadecimal' string, return next pos in buffer */
 char *md5sumToString(char *buf, const char *last, const uint8 md5sum[16]);
 
-/**
- * Only allow certain keys. You can define the filter to be used. This makes
- *  sure no invalid keys can get into an editbox, like BELL.
- * @param key character to be checked
- * @param afilter the filter to use
- * @return true or false depending if the character is printable/valid or not
- */
 bool IsValidChar(WChar key, CharSetFilter afilter);
 
 size_t Utf8Decode(WChar *c, const char *s);
@@ -167,7 +88,8 @@ static inline WChar Utf8Consume(const char **s)
 }
 
 
-/** Return the length of a UTF-8 encoded character.
+/**
+ * Return the length of a UTF-8 encoded character.
  * @param c Unicode character.
  * @return Length of UTF-8 encoding for character.
  */
@@ -222,6 +144,30 @@ static inline char *Utf8PrevChar(char *s)
 	return ret;
 }
 
+size_t Utf8StringLength(const char *s);
+
+/**
+ * Is the given character a text direction character.
+ * @param c The character to test.
+ * @return true iff the character is used to influence
+ *         the text direction.
+ */
+static inline bool IsTextDirectionChar(WChar c)
+{
+	switch (c) {
+		case CHAR_TD_LRM:
+		case CHAR_TD_RLM:
+		case CHAR_TD_LRE:
+		case CHAR_TD_RLE:
+		case CHAR_TD_LRO:
+		case CHAR_TD_RLO:
+		case CHAR_TD_PDF:
+			return true;
+
+		default:
+			return false;
+	}
+}
 
 static inline bool IsPrintable(WChar c)
 {
@@ -240,19 +186,16 @@ static inline bool IsPrintable(WChar c)
  */
 static inline bool IsWhitespace(WChar c)
 {
-	return
-	  c == 0x0020 /* SPACE */ ||
-	  c == 0x3000 /* IDEOGRAPHIC SPACE */
-	;
+	return c == 0x0020 /* SPACE */ || c == 0x3000; /* IDEOGRAPHIC SPACE */
 }
 
 /* Needed for NetBSD version (so feature) testing */
-#ifdef __NetBSD__
+#if defined(__NetBSD__) || defined(__FreeBSD__)
 #include <sys/param.h>
 #endif
 
 /* strndup is a GNU extension */
-#if defined(_GNU_SOURCE) || (defined(__NetBSD_Version__) && 400000000 <= __NetBSD_Version__)
+#if defined(_GNU_SOURCE) || (defined(__NetBSD_Version__) && 400000000 <= __NetBSD_Version__) || (defined(__FreeBSD_version) && 701101 <= __FreeBSD_version)
 #	undef DEFINE_STRNDUP
 #else
 #	define DEFINE_STRNDUP
@@ -267,4 +210,6 @@ char *strndup(const char *s, size_t len);
 char *strcasestr(const char *haystack, const char *needle);
 #endif /* strcasestr is available */
 
+int strnatcmp(const char *s1, const char *s2);
+
 #endif /* STRING_FUNC_H */
